// ActivaAPI.cpp : Defines the entry point for the DLL application.
//
#pragma once 

#include <windows.h>
#include <stdlib.h>
#include <string>
#include <map>
#include <sstream>
#include <map>
#include <vector>
#include <set>

/* do not touch this: it is automatically updated by SVN in every commit
BUT you need to *touch* this file in order to force a commit and update svn revision flag.
*/    
#define	SVN_REVISION	"$Revision: 549 $"

#ifdef _WIN32
#   ifdef ACTIVAAPI_EXPORTS
#       define LIBRARY_API_C extern "C" __declspec(dllexport)
#		define LIBRARY_API_CPP __declspec(dllexport)
#   else
#       define LIBRARY_API_C extern "C" __declspec(dllimport)
#		define LIBRARY_API_CPP __declspec(dllimport)
#   endif
#elif
#    define LIBRARY_API
#endif


/*
 * http://groups.google.es/group/microsoft.public.vc.stl/browse_frm/thread/1dd29dee6dc36b62/22d3a6e4a0461166?lnk=st&q=stl+std%3A%3Astring+format+integer&rnum=2&hl=es#22d3a6e4a0461166
 */
template <class TYPE> std::string ToString( const TYPE & t ) { 
	std::ostringstream os; 
	os << t; 
	return os.str(); 
} 

/// cross reference for open monitors (opaque handler)
typedef	int MonitorXref;

typedef	std::set<int> Set_xref ;				//< set of monitor xref

/// agent status
typedef enum {
	LOGIN = 1,		
	LOGOUT = 2,
	READY = 3,
	NOTREADY = 4
} AGENT_STATUS ;

typedef enum {
	ast_cause_normal = 0,
	ast_cause_active = 1,
	ast_cause_new_call = 2,
	ast_cause_abandon_call = 3,
	ast_cause_held = 4,
	ast_cause_unknown = -1,
	ast_cause_out_of_order = -2,
	ast_cause_user_busy = -3,
	ast_cause_unallocated = -4,		// wrong number
	ast_cause_alerting_noanswer = -5,	// no answer or calling to movile out of coverage
	ast_cause_reject = -6,		//call rejected 
	ast_cause_unreachable = -7,	// no route to destination
	ast_cause_forwarded = -8,		// a call is forwarded by the switch
	ast_cause_temp_failure = -9,	// temporary failure of the net
	ast_cause_congestion = -10,  // network congestion
	ast_cause_answering_machine = -11, //Automatic dialplan hangup with cause answering machine detected
	ast_cause_modem_fax = -12 //Automatic dialplan hangup modem fax detected
} call_cause;

/**
 * interface types
 */
typedef enum {
	not_enough_devices = -14,
	no_such_channel = -13,
	interface_not_found = -12,
	not_logged_in = -11,
	no_such_queue = -10,
	interface_not_specified = -9,
	queue_not_specified = -8,
	already_logged_in = -7,
	error_not_implemented = -6,
	fails_already_connected = -5,
	invalid_callid = -4,
	socket_error = -3,
	time_out = -2,
	generic_failure = -1,
	ok = 0,
	ok_feature_dialed = 1
} AC_RESULT;

typedef enum {
	at_plaintext = 0,
	at_md5 = 1
} AUTH_TYPE;

/*
 * ManagerEvent
 */
class	ManagerEvent
{
public:
	// call model
	std::string	m_address;

	// asterisk
    std::string m_uniqueId;		/// uniqueId of event
    std::string m_destUniqueId;	/// destination uniqueId of event
    std::string m_srcUniqueId;	/// source uniqueId of event
	std::string m_context;		/// context of event
    std::string m_extension;	/// extension of event
    std::string m_application;	/// application of event
    std::string m_appData;		/// application data of event
    std::string m_priority;		/// priority of event
    std::string m_channel;		/// channel of event
	std::string m_from;			/// UnparkedCall
    std::string m_oldname;		/// rename event
    std::string m_newname;		/// rename event
	std::string	m_state;		/// state of event
	std::string	m_callerId;		/// callerId of event
	std::string	m_callerIdName;	/// callerId name of event
	std::string	m_connectedLineNum;	///connectedLineNum of event, new in asterisk 1.8.5.0
	std::string	m_connectedLineName;///connectedLineName of event, new in asterisk 1.8.5.0
	std::string m_cause;		/// cause of event
	std::string m_causeTxt;		/// cause TXT of event
	std::string	m_queue;		/// queue of event
	std::string	m_count;		/// number of clients in queue
	std::string	m_position;		/// position of client in queue
	std::string m_origposition;	/// Original position of client in queue Asterisk 1.4
	std::string m_actionId;		/// identifier of action
	std::string m_reason;		/// reason of event
	std::string m_response;		/// response of originate Asterisk 1.4
	std::string m_holdtime;		/// time in hold Asterisk 1.4
	std::map<std::string, std::string> m_attributes;	/// map of attributes
};	

/// Logger interface: any customized logger must implement this interface
class	ASTLogListener {
public:
	virtual void	LogTrace0(const std::string & sOrigin, const std::string & sMsg) = 0;
	virtual void	LogTrace(const std::string & sOrigin, const std::string & sMsg) = 0;
	virtual void	LogInfo(const std::string & sOrigin, const std::string & sMsg) = 0;
	virtual void	LogWarn(const std::string & sOrigin, const std::string & sMsg) = 0;
	virtual void	LogError(const std::string & sOrigin, const std::string & sMsg) = 0;
};

/**
 * When ASTCstaProvider triggers an event it does so by calling a method on this listener class.
 * So applications usually implement their own ASTCstaListener-derived class and override OnXXX() methods.
 *
 * Application should implement this interface deriving a class from ASTCstaListener and re-implementing the needed methods.
 * Listener interface methods are supposed to: 
 *	-# Do not change parameters passed. 
 *	-# Be short, fast to avoid blocking the provider thread.
 *	-# Do not call any provider method
 * 
 * Common parameters:
 *	-# MonitorXref xref is the monitor cross-reference opaque handler returned in OpenMonitor() call
 *	-# DWORD callId is a call identifier. It is generated by ASTCstaProvider and not directly related to Asterisk uniqueid's
 *	-# device: address subject of event. 
 *	-# callerId/callerIdName: cid/cidname provided by Asterisk, if any.
 */
/// CSTA event listener
class	ASTCstaListener {
public:
	/// a call with callId has been originated at originatingDevice
	virtual void	OnOriginated ( MonitorXref xref, DWORD callId, std::string originatingDevice, std::string callerId, std::string callerIdName, std::string calledDevice, std::string userData, call_cause cause) = 0;
	
	/// a call with callId is alerting at alertingDevice
	virtual void	OnAlerting ( MonitorXref xref, DWORD callId, std::string alertingDevice, std::string callerId, std::string callerIdName, std::string calledDevice, std::string lastRedirectingDevice, std::string userData, call_cause cause) = 0;

	/// a call with callId has been established at establishedDevice
	virtual void	OnEstablished ( MonitorXref xref, DWORD callId, std::string establishedDevice, std::string callerId, std::string callerIdName, std::string calledDevice, std::string lastRedirectingDevice, std::string userData, call_cause cause) = 0;

	/// a call with callId has been failed at failingDevice
	virtual void	OnFailed ( MonitorXref xref, DWORD callId, std::string failingDevice, std::string callerId, std::string callerIdName, std::string calledDevice, std::string lastRedirectionDevice, std::string userData, call_cause cause) = 0;

	/// a call with callId has been queued at queue
	virtual void	OnQueued ( MonitorXref xref, DWORD callId, std::string queuedDevice, std::string queue, int position_in_queue, int queue_len, std::string callerId, std::string callerIdName, std::string calledDevice, std::string userData, call_cause cause) = 0;

	/// a call with callId has been diverted from a device (example: from a queue)
	virtual void	OnDiverted ( MonitorXref xref, DWORD callId, std::string divertedDevice, std::string divertingDevice, std::string callerId, std::string callerIdName, std::string calledDevice, std::string userData, call_cause cause) = 0;

	/// a call with callId in releasingDevice has been hung-up 
	virtual void	OnConnectionCleared ( MonitorXref xref, DWORD callId, std::string releasingDevice, call_cause cause) = 0;

	/// a call with callId has been completely destroyed 
	virtual void	OnCallCleared ( MonitorXref xref, DWORD callId, std::string userData, call_cause cause) = 0;

	/// provider link-down handler
	virtual void	OnLinkDown() = 0;

	// a call with callId has been holding by holdingDevice
	virtual void	OnHold ( MonitorXref xref, DWORD callId, std::string holdingDevice, call_cause cause) = 0;

	virtual void	OnUnhold ( MonitorXref xref, DWORD callId, std::string unholdingDevice, call_cause cause) = 0;

};

/// ASTProvider listener interface 
/**
 * ASTListener defines a listener interface that will receive
 * ASTProvider event notifications.
 * Application should implement this interface deriving a class from ASTListener and
 * re-implementing the needed methods.
 *
 * Listener interface methods are supposed to: 
 *	-# Do not change parameters passed. 
 *	-# Be short, fast to avoid blocking the provider thread.
 *	-# Do not call any provider method
 */
class	ASTListener {
public:
	// manager api interface
	virtual void	OnNewStateEvent(ManagerEvent ev) = 0;
	virtual void	OnNewCallerIdEvent(ManagerEvent ev) = 0;
	virtual void	OnNewExten(ManagerEvent ev) = 0;
	virtual void	OnChannelEvent(ManagerEvent ev) = 0;
	virtual void	OnHangup(ManagerEvent ev) = 0;
	virtual void	OnNewChannelEvent(ManagerEvent ev) = 0;
	virtual void	OnDialEvent(ManagerEvent ev) = 0;
	virtual void	OnRenameEvent(ManagerEvent ev) = 0;
	virtual void	OnLinkEvent(ManagerEvent ev) = 0;
	virtual void	OnUnlinkEvent(ManagerEvent ev) = 0;
	virtual void	OnJoinEvent(ManagerEvent ev) = 0;
	virtual void	OnLeaveEvent(ManagerEvent ev) = 0;
	virtual void	OnLinkDown() = 0;
	virtual void	OnUnparkedCallEvent(ManagerEvent ev) = 0;
	virtual void	OnOriginateFailureEvent(ManagerEvent ev) = 0;
	virtual void	OnQueueCallerAbandonEvent(ManagerEvent ev) = 0;
	virtual void	OnHoldEvent(ManagerEvent ev) = 0;
	virtual void	OnUnholdEvent(ManagerEvent ev) = 0;
};

/**
 * This is an interface to the main class of Activa framework ASTCstaProvider. 
 * Provides a csta-like programming interface.
 * To create an instance you can call CreateASTCSTAProvider factory method 
 * Events are reported through a listener class implementing ASTCstaListener interface.
 * A custom logger can be attached via ASTLogListener
 */

/// This is the main class of Activa framework. Provides a csta-like programming interface.
class	ASTCstaProviderInterface : public ASTListener 
{
public:
	/// factory method
	//virtual static ASTCstaProviderInterface* Create(ASTCstaListener	*listener, ASTLogListener *logger) = 0;
	//	~ASTCstaProviderInterface();

	/// Connect to Asterisk 
	virtual AC_RESULT	Connect(std::string	sHost, int nPort, std::string user, std::string pwd, AUTH_TYPE eAuthType = at_md5) = 0;

	/// Disconnect from Asterisk
	virtual bool		Disconnect()=0;

	/// Opens a monitor to a device. Monitors are refererd by an opaque int handler known as monitor xref 
	/// (monitor cross-reference)
	virtual MonitorXref	OpenMonitor(std::string	deviceId)=0;

	/// Close a previously opened monitor
	virtual bool		CloseMonitor(MonitorXref xref)=0;

	/// Originates a call from callingDevice to calleddevice using 'context'.
	virtual AC_RESULT	MakeCall(std::string callingDevice, std::string contextCalling, std::string calledDevice, DWORD &callid, std::string contextCalled, std::string callerIdName, std::string userData, std::string variable, bool bLockReaderAfterResponse = false)=0;

	///	Originate a predictive call from callingDevice to calledDevice using 'context'
	virtual AC_RESULT	MakePredictiveCall(std::string callingDevice, std::string contextCalling, std::string calledDevice, std::string contextCalled, DWORD &callid, std::string callerId, std::string userData, DWORD timeout, std::string variable, bool bLockReaderAfterResponse = false)=0;

	/// Disconnects a call identified by uniqueid/callid
	virtual AC_RESULT	ClearCall(DWORD callId)=0;

	/// Disconnects a Connection identified by callId and device)
	virtual AC_RESULT	ClearConnection(DWORD callid, std::string address)=0;

	/// Redirects a call to an extsnsion
	virtual AC_RESULT	FastTransfer(DWORD callId, std::string transferringDevice, std::string destination, std::string destinationContext)=0;

	/// Sets Do-not-disturb flag on/off
	virtual AC_RESULT	SetDoNotDisturb(std::string deviceId, bool bDND_on, std::string dndContext)=0;

	/// Set agentState
	virtual AC_RESULT	SetAgentStatus(std::string deviceId, std::string queue, AGENT_STATUS newStatus, std::string agentContext)=0;

	// Hold a call
	virtual AC_RESULT	HoldCall(DWORD callId, std::string holdingDevice, std::string holdContext)=0;

	// Retrieve call
	virtual AC_RESULT	RetrieveCall(DWORD callId, std::string retreivingDevice, std::string retrieveContext)=0;
	
	/// Start Recording
	virtual AC_RESULT	StartRecording(DWORD callid, std::string address, std::string RecordingFileName, std::string format, std::string mix)=0;

	/// Stop Recording
	virtual AC_RESULT	StopRecording(DWORD callid, std::string address)=0;

	/// Pause Recording
	virtual AC_RESULT	PauseRecording(DWORD callid, std::string address)=0;

	/// Unpause Recording
	virtual AC_RESULT	UnpauseRecording(DWORD callid, std::string address)=0;

	/// Change Recording File Name
	virtual AC_RESULT	ChangeRecordingFileName(DWORD callid, std::string address, std::string NewRecordingFileName)=0;

	/**
	*	\return New Callid
	*	This method is implemented ONLY to allow ASTTapiProvider::TSPI_lineMakeCall() to get a real unique callid before Originate(). 
	*/
	virtual DWORD		GetNewCallId() =0;

	/**
	*	\return result
	*	This method is implemented ONLY to allow ASTTapiProvider::TSPI_lineMakeCall() to drop a idCall previously acquired through GetCallId()
	*/
	virtual bool		DropCallId(DWORD idCall) =0;

	/**
	*	returns the monitor set linked to the given callid
	*
	**/
	virtual bool	GetMonitorSet(DWORD callId, Set_xref & setXref) =0;

	/**
	*	UnLocks Reader Thread
	*
	**/
	virtual void		UnLockReader() =0;

};


LIBRARY_API_C	ASTCstaProviderInterface*	CreateASTCSTAProvider (ASTCstaListener *listener,ASTLogListener *logger);
LIBRARY_API_C	void						DeleteASTCSTAProvider (ASTCstaProviderInterface* cstaprovider);
/// Gets address from channel specification
LIBRARY_API_CPP	std::string					AsteriskCall_GetAddressFromChannel (std::string channel);
/// Gets dialable number from Channel
LIBRARY_API_CPP	std::string					AsteriskCall_GetDialableNumberFromChannel (std::string channel);
/// true if device is void or equal to \<unknown>
LIBRARY_API_CPP	bool						AsteriskCall_IsUnknown (std::string device);

//type definitions used to load the dll at runtime. 
typedef	ASTCstaProviderInterface*	(*LPFN_CreateASTCSTAProvider) (ASTCstaListener *listener,ASTLogListener *logger);
typedef void						(*LPFN_DeleteASTCSTAProvider) (ASTCstaProviderInterface* cstaprovider);

/*
cross platform run time load exemple (not tested yet)

#ifndef USE_PRECOMPILED_HEADERS
#ifdef WIN32
#include <direct.h>
#include <windows.h>
#else
#include <sys/types.h>
#include <dlfcn.h>
#endif
#include <iostream>
#endif

    using namespace std;

#ifdef WIN32
    HINSTANCE lib_handle;
#else
    void *lib_handle;
#endif

    // Where retType is the pointer to a return type of the function
    // This return type can be int, float, double, etc or a struct or class.

    typedef retType* func_t;  

    // load the library -------------------------------------------------
#ifdef WIN32
    string nameOfLibToLoad("C:\opt\lib\libctest.dll");
    lib_handle = LoadLibrary(TEXT(nameOfLibToLoad.c_str()));
    if (!lib_handle) {
        cerr << "Cannot load library: " << TEXT(nameOfDllToLoad.c_str()) << endl;
    }
#else
    string nameOfLibToLoad("/opt/lib/libctest.so");
    lib_handle = dlopen(nameOfLibToLoad.c_str(), RTLD_LAZY);
    if (!lib_handle) {
        cerr << "Cannot load library: " << dlerror() << endl;
    }
#endif

...
...
...

    // load the symbols -------------------------------------------------
#ifdef WIN32
    func_t* fn_handle = (func_t*) GetProcAddress(lib_handle, "superfunctionx");
    if (!fn_handle) {
        cerr << "Cannot load symbol superfunctionx: " << GetLastError() << endl;
    }
#else
    // reset errors
    dlerror();

    // load the symbols (handle to function "superfunctionx")
    func_t* fn_handle= (func_t*) dlsym(lib_handle, "superfunctionx");
    const char* dlsym_error = dlerror();
    if (dlsym_error) {
        cerr << "Cannot load symbol superfunctionx: " << dlsym_error << endl;
    }
#endif

...
...
...

    // unload the library -----------------------------------------------

#ifdef WIN32
    FreeLibrary(lib_handle);
#else
    dlclose(lib_handle);
#endif
      

*/