/*
	Name: ActivaTSP.cpp
	Copyright: Under the Mozilla Public License Version 1.1 or later
	Author: ACTIVA
	Date: MAY/06
*/

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is astprovider.
 *
 * The Initial Developer of the Original Code is ACTIVA.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): none
 *
 * ***** END LICENSE BLOCK ***** */

#include <windows.h>

#include "asttapiprovider.h"
#include "activatsp_helper.h"
#include "config.h"


#define MIN(a, b) (a < b ? a : b)
#define	DEFAULT_FILE_SIZE	(4*1000*1000)		// 4 MB
#define DEFAULT_LOG_FILES	10
#define DEFAULT_MAX_BUFFER_SIZE  (60*1000)

/*
 * globals
 */
ASTTapiProvider	*g_xTapiProvider = NULL;
BasicLogger		g_xLogger(registryGetLogFilePath(DEFAULT_LOG_FILE_PATH), registryGetMaxLogFiles(DEFAULT_LOG_FILES), registryGetLogFileSize(DEFAULT_FILE_SIZE), registryGetMaxBufferSize(DEFAULT_MAX_BUFFER_SIZE));
HPROVIDER		g_hProvider ;
HINSTANCE		g_hinst = 0;
std::vector<std::string> g_vOutDnVector;   // vector with dn from expresion

BOOL APIENTRY DllMain( HINSTANCE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    if( ul_reason_for_call == DLL_PROCESS_ATTACH ) {
        g_hinst = hModule;
	}

    return TRUE;
}


/**
 * following code based on asttapi developed by Nick Knight
 * asttapi.sourceforge.org
 *
 *	Function TSPI_providerEnumDevices
 *	TAPI calls the TSPI_providerEnumDevices function before TSPI_providerInit to determine 
 *  the number of line and phone devices supported by the service provider. 
 *	\see http://msdn2.microsoft.com/en-us/library/ms725957.aspx
 *	\param dwPermanentProviderID
 *	\param lpdwNumLines
 *	\param lpdwNumPhones
 *	\param hProvider
 *	\param lpfnLineCreateProc
 *	\param lpfnPhoneCreateProc
 */
LONG TSPIAPI TSPI_providerEnumDevices(
		DWORD dwPermanentProviderID,
		LPDWORD lpdwNumLines,
		LPDWORD lpdwNumPhones,
		HPROVIDER hProvider,
		LINEEVENT lpfnLineCreateProc,
		PHONEEVENT lpfnPhoneCreateProc)
{
	g_hProvider = hProvider;

	registryInit();

	g_xLogger.EnableTraceToFile(registryIsTraceToFile(0));
	g_xLogger.SetTraceLevel( registryGetLogLevel(TRACE_LEVEL_ERROR | TRACE_LEVEL_WARNING | TRACE_LEVEL_INFO | TRACE_LEVEL_TRACE) );

	g_xLogger.LogTrace("TSP", "TSPI_providerEnumDevices()");

	DWORD n=ASTTapiProvider::ProcessDnExp(g_vOutDnVector,g_xLogger);
	*lpdwNumLines = n;
	*lpdwNumPhones = 0;
	// won't create line nor phones on the fly so we do not need to store lpfnXXXCreateProcs
	return 0;
}

/**
 * TSPI_providerInit
 *	Initializes ASTTapiProvider object
 *	\see http://msdn2.microsoft.com/en-us/library/ms725960.aspx
 *	\param dwTSPIVersion
 *	\param dwPermanentProviderId
 *	\param dwLineDeviceIDBase
 *	\param dwNumLines
 *	\param dwNumPhones
 *	\param lpfnCompletionProc
 *	\param lpdwTSPIOptions
 */
LONG TSPIAPI TSPI_providerInit(
    DWORD               dwTSPIVersion,
    DWORD               dwPermanentProviderID,
    DWORD               dwLineDeviceIDBase,
    DWORD               dwPhoneDeviceIDBase,
    DWORD_PTR           dwNumLines,
    DWORD_PTR           dwNumPhones,
    ASYNC_COMPLETION    lpfnCompletionProc,
    LPDWORD             lpdwTSPIOptions                         // TSPI v2.0
    )
{
	registryInit();

	g_xLogger.EnableTraceToFile(registryIsTraceToFile(0));
	g_xLogger.SetTraceLevel( registryGetLogLevel(TRACE_LEVEL_ERROR | TRACE_LEVEL_WARNING | TRACE_LEVEL_INFO | TRACE_LEVEL_TRACE) );

	g_xLogger.LogTrace("TSP", "TSPI_providerInit()");
    
	g_xTapiProvider = ASTTapiProvider::Create (	&g_xLogger,
												lpfnCompletionProc, // tapi callback function  
												dwPermanentProviderID,
												dwLineDeviceIDBase,
												g_vOutDnVector      //Dn vector
											   );

	DWORD n = ASTTapiProvider::ProcessDnExp(g_vOutDnVector, g_xLogger);

	dwNumLines = n;
    dwNumPhones = 0;
    
	// LINETSPIOPTION_NONREENTRANT  see http://msdn.microsoft.com/en-us/library/ms725960(VS.85).aspx
	DWORD	dwLINETSPIOPTION_NONREENTRANT = 0;
	registryGet("LINETSPIOPTION_NONREENTRANT", dwLINETSPIOPTION_NONREENTRANT);
	if (dwLINETSPIOPTION_NONREENTRANT != 0) {
		*lpdwTSPIOptions = LINETSPIOPTION_NONREENTRANT;
		g_xLogger.LogTrace("TSP", "WARNING: flag LINETSPIOPTION_NONREENTRANT is set to YES");
	}

	return 0;
}


/**
 * Function TSPI_providerShutdown
 * Shutdown and clean up.
 *	\see http://msdn2.microsoft.com/en-us/library/ms725963.aspx
 *	\param dwTSPIVersion
 *	\param dwPermanentProviderId
 */
LONG TSPIAPI TSPI_providerShutdown(
    DWORD               dwTSPIVersion,
    DWORD               dwPermanentProviderID                   // TSPI v2.0
    )
{

	g_xLogger.LogTrace("TSP", "TSPI_providerShutdown()");
	
	WSACleanup();
	
	if (g_xTapiProvider!=NULL) { 
		g_xTapiProvider->TSPI_providerShutdown(dwTSPIVersion, dwPermanentProviderID);
		delete g_xTapiProvider;
		g_xTapiProvider = NULL;
	}
	
	return 0;
}


/**
 * Function TSPI_lineNegotiateTSPIVersion
 *	\param dwDeviceId
 *	\param dwLowVersion
 *	\param dwHighVersion
 *	\param lpdwTSPIVersion
 */
LONG TSPIAPI TSPI_lineNegotiateTSPIVersion(
	DWORD dwDeviceID,
	DWORD dwLowVersion,
	DWORD dwHighVersion,
	LPDWORD lpdwTSPIVersion)
{
	LONG tr = 0;

	if ( dwLowVersion <= TAPI_CURRENT_VERSION ) {
		*lpdwTSPIVersion = MIN(TAPI_CURRENT_VERSION,dwHighVersion);
	}
	else {
		tr = LINEERR_INCOMPATIBLEAPIVERSION;
	}

	return tr;
}


/**
 * Function TSPI_lineGetDevCaps
 * \see http://msdn.microsoft.com/library/default.asp?url=/library/en-us/tapi/tspi/tspi_service_provider_functions.asp
 * Allows TAPI to check our line capabilities before placing a call
*	\param dwDeviceId
 *	\param dwTSPIVersion
 *	\param dwExtVersion
 *	\param pldc
 */
LONG TSPIAPI TSPI_lineGetDevCaps(
    DWORD           dwDeviceID,
    DWORD           dwTSPIVersion,
    DWORD           dwExtVersion,
    LPLINEDEVCAPS   pldc
    )
{ 
	if (g_xTapiProvider==NULL) {	// not initialized
		return LINEERR_RESOURCEUNAVAIL ;
	}
 
	return g_xTapiProvider->TSPI_lineGetDevCaps(dwDeviceID, dwTSPIVersion, dwExtVersion, pldc);
}

/**
 * Function TSPI_lineGetAddressCaps
 * Allows TAPI to check our line capabilities before placing a call
 *	\param dwDeviceID
 *	\param dwAddressID
 *	\param dwTSPIVersion
 *	\param dwExtVersion
 *	\param pac
 */
LONG TSPIAPI TSPI_lineGetAddressCaps(
    DWORD   dwDeviceID,
    DWORD   dwAddressID,
    DWORD   dwTSPIVersion,
    DWORD   dwExtVersion,
    LPLINEADDRESSCAPS  pac)
{
	if (g_xTapiProvider==NULL) {
		return LINEERR_NOMEM ;
	}
	return g_xTapiProvider->TSPI_lineGetAddressCaps(dwDeviceID, dwAddressID, dwTSPIVersion, dwExtVersion, pac);
}


/**
 * Function TSPI_lineOpen
 *	This function is typically called where the software needs to reserve some 
 * hardware, you can assign any 32bit value to the *phdLine, and it will be sent
 * back to any future calls to functions about that line.
 *	Becuase this is sockets not hardware we can set-up the sockets required in this 
 * functions, and perhaps get the thread going to read the output of the manager.
 *	\param dwDeviceId
 *	\param htLine
 *	\param phdLine
 *	\param dwTSPIVersion
 *	\param pfnEventProc
 */
LONG TSPIAPI TSPI_lineOpen(
    DWORD               dwDeviceID,
    HTAPILINE           htLine,
    LPHDRVLINE          phdLine,
    DWORD               dwTSPIVersion,
    LINEEVENT           pfnEventProc
    )
{
	if (g_xTapiProvider==NULL) {
		return LINEERR_NOMEM ;
	}

	return g_xTapiProvider->TSPI_lineOpen(dwDeviceID, htLine, phdLine, dwTSPIVersion, pfnEventProc);
}


/**
* Function TSPI_lineClose
* Called by TAPI when a line is no longer required.
* TSPI_lineClose: 
*	The TSPI_lineClose function closes the specified open line device after completing 
*	or aborting all outstanding calls and asynchronous operations on the device.
* \see http://windowssdk.msdn.microsoft.com/en-us/library/ms725531.aspx
* \param hdCall
*
*/
LONG TSPIAPI TSPI_lineClose(HDRVLINE hdLine)
{
	if (g_xTapiProvider==NULL) {
		return LINEERR_NOMEM ;
	}

	return g_xTapiProvider->TSPI_lineClose(hdLine);
}


/**
 * Function TSPI_lineMakeCall
 *	This function is called by TAPI to initialize a new outgoing call. This will
 * initiate the call and return, when the call is made (but not necasarily connected)
 * we should then signal TAPI via the asyncrounous completion function.
 *
 * \see http://msdn.microsoft.com/library/default.asp?url=/library/en-us/tapi/tspi/tspi_linemakecall.asp
 *	\param dwRequestId
 *	\param hdline
 *	\param htCall
 *	\param phdCall
 *	\param pszDestAddress
 *	\param dwCountrycode
 *	\param pCallParams
 */
LONG TSPIAPI TSPI_lineMakeCall(
    DRV_REQUESTID       dwRequestID,
    HDRVLINE            hdLine,
    HTAPICALL           htCall,
    LPHDRVCALL          phdCall,
    LPCWSTR             pszDestAddress,
    DWORD               dwCountryCode,
    LPLINECALLPARAMS    const pCallParams
    )
{
	if (g_xTapiProvider==NULL) {
		return LINEERR_NOMEM ;
	}

	return g_xTapiProvider->TSPI_lineMakeCall(dwRequestID, hdLine, htCall, phdCall, pszDestAddress, dwCountryCode, pCallParams);
}


/**
 * Function TSPI_lineDrop
 * 
 * This function is called by TAPI to signal the end of a call. The status 
 *	information for the call should be retained until the function TSPI_lineCloseCall
 *	is called.
 *
 * The TSPI_lineDrop function drops or disconnects the specified call. User-user information 
 *	can optionally be transmitted as part of the call disconnect. This function can be 
 *	called by the application at any time. When TSPI_lineDrop returns, the call should be idle.
 */
LONG TSPIAPI TSPI_lineDrop(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    LPCSTR              lpsUserInfo,
    DWORD               dwSize
    )
{
	if (g_xTapiProvider==NULL) {
		return LINEERR_NOMEM ;
	}

	return g_xTapiProvider->TSPI_lineDrop(dwRequestID, hdCall, lpsUserInfo, dwSize );
}

/**
 * TSPI_lineCloseCall:
 *	The TSPI_lineCloseCall function deallocates the call after completing or aborting 
 *	all outstanding asynchronous operations on the call.
 *
 */
LONG TSPIAPI TSPI_lineCloseCall(HDRVCALL hdCall)
{
	if (!g_xTapiProvider) {	// do nothing
		return 0;		
	}
	return g_xTapiProvider->TSPI_lineCloseCall(hdCall);
}



/*
*
* Status
*
* if TAPI requires to find out the status of our lines then it can call
* the following functions.
*/

/*
* Function TSPI_lineGetLineDevStatus
* Currently not implemented
*
*/
LONG TSPIAPI TSPI_lineGetLineDevStatus(
    HDRVLINE            hdLine,
    LPLINEDEVSTATUS     plds
    )
{
	g_xLogger.LogTrace("TSP", "TSPI_lineGetLineDevStatus VOID IMPLEMENTATION");
    return (0);
}


/*
* Function TSPI_lineGetAdressStatus
* Allows an application to query the specified address for its current status.
* Currently not implemented	
* \see http://msdn2.microsoft.com/en-us/library/ms735683.aspx
*/
LONG TSPI_lineGetAdressStatus(
    HDRVLINE hdLine,
    DWORD dwAddressID,
    LPLINEADDRESSSTATUS pas)
{
	g_xLogger.LogTrace("TSP", "TSPI_lineGetAdressStatus");
    return (0);
}

/**
 * Function TSPI_lineGetCallStatus
 * 
 * \see http://windowssdk.msdn.microsoft.com/en-us/library/ms735544.aspx
 */
LONG TSPIAPI TSPI_lineGetCallStatus(
    HDRVCALL            hdCall,
    LPLINECALLSTATUS    pls
    )
{
	g_xLogger.LogTrace("TSP", "TSPI_lineGetCallStatus");
    return (0);
}



/**
* Function TSPI_lineGetCallInfo
*
* As the function says.
* \see http://windowssdk.msdn.microsoft.com/en-us/library/ms725566.aspx
*/
LONG TSPIAPI TSPI_lineGetCallInfo(
    HDRVCALL            hdCall,
    LPLINECALLINFO      lpCallInfo
    )
{
	if (!g_xTapiProvider) {
		return LINEERR_OPERATIONFAILED;
	}

	return g_xTapiProvider->TSPI_lineGetCallInfo(hdCall, lpCallInfo);
}



/**
* Function TSPI_lineGetCallAddressID
*
* As the function says.
*
*/
LONG TSPIAPI TSPI_lineGetCallAddressID(
    HDRVCALL            hdCall,
    LPDWORD             pdwAddressID
    )
{
	g_xLogger.LogTrace("TSP", "TSPI_lineGetCallAddressID VOID IMPLEM. hdCall="+ToString(hdCall));
    return (0);
}
////////////////////////////////////////////////////////////////
//
// Installation, removal and configuration
//
////////////////////////////////////////////////////////////////////////////////
/**
 * Function TUISPI_providerInstall
 *	\see http://msdn2.microsoft.com/en-us/library/ms725984.aspx
 */
LONG TSPIAPI TUISPI_providerInstall(
    TUISPIDLLCALLBACK pfnUIDLLCallback,
    HWND hwndOwner,
    DWORD dwPermanentProviderID)
{
	registryInit();
	registrySetHostIp(registryGetHostIp("localhost"));
	registrySetHostPort(registryGetHostPort(3010));

	//MessageBox(hwndOwner, "Activa TAPI Service Provider installed", "Startup", MB_OK);

    return (0);
}

LONG
TSPIAPI
TSPI_providerInstall(
    HWND                hwndOwner,
    DWORD               dwPermanentProviderID
    )
{
	return (0);
}


/**
* Function TUISPI_providerRemove
*
* When TAPI is requested to remove the TSP via the call lineRemoveProvider.
*	\see http://msdn2.microsoft.com/en-us/library/ms725985.aspx
*
*/
LONG TSPIAPI TUISPI_providerRemove(
    TUISPIDLLCALLBACK pfnUIDLLCallback,
    HWND hwndOwner,
    DWORD dwPermanentProviderID)
{
    return (0);
}


LONG TSPIAPI TSPI_providerRemove(
    HWND    hwndOwner,
    DWORD   dwPermanentProviderID)
{
    return (0);
}
////////////////////////////////////////////////////////////////////////////////
// Function TUISPI_providerRemove
//
// When TAPI is requested to remove the TSP via the call lineRemoveProvider.
//
////////////////////////////////////////////////////////////////////////////////
LONG TSPIAPI TUISPI_providerUIIdentify(
    TUISPIDLLCALLBACK pfnUIDLLCallback,
    LPWSTR pszUIDLLName)
{
	char szPath[MAX_PATH+1];
	GetModuleFileNameA(g_hinst,szPath,MAX_PATH);
	errno_t err;
	size_t sizePszUIDLLName;
	err = mbstowcs_s(&sizePszUIDLLName,pszUIDLLName,strlen(szPath)+1,szPath,strlen(szPath)+1);
	return (0);
}

////////////////////////////////////////////////////////////////////////////////
//
// Dialog config area
//
////////////////////////////////////////////////////////////////////////////////

/**
 * Function TUISPI_lineConfigDialog
 *
 * Called when a request for config is made upon us.
 *	\see http://msdn2.microsoft.com/en-us/library/ms725976.aspx
 */
LONG TSPIAPI TUISPI_lineConfigDialog(                                        // TSPI v2.0
    TUISPIDLLCALLBACK   lpfnUIDLLCallback,
    DWORD               dwDeviceID,
    HWND                hwndOwner,
    LPCWSTR             lpszDeviceClass
    )
{
	DialogBox(g_hinst,
        MAKEINTRESOURCE(IDD_DIALOG),
        hwndOwner,
        (DLGPROC)ConfigDlgProc);

	return (0);
}

/**
// Function TSPI_providerUIIdentify
//
// Becuase the UI part of a TSP can exsist in another DLL we need to tell TAPI
// that this is the DLL which provides the UI functionality as well.
//
*/
LONG TSPIAPI TSPI_providerUIIdentify(                                        // TSPI v2.0
        LPWSTR              lpszUIDLLName
    )
{
	char szPath[MAX_PATH+1];
	GetModuleFileNameA(g_hinst,szPath,MAX_PATH);
	size_t sizeLpszUIDLLName;
	errno_t err = mbstowcs_s(&sizeLpszUIDLLName,lpszUIDLLName,strlen(szPath)+1,szPath,strlen(szPath)+1);

	return (0);
}

LONG
TSPIAPI
TSPI_providerGenericDialogData(                                 // TSPI v2.0
    DWORD_PTR           dwObjectID,
    DWORD               dwObjectType,
    LPVOID              lpParams,
    DWORD               dwSize
    )
{
	return (0);
}
/**
 * Function TUISPI_providerConfig
 * This function is called when juser configures the tsp from control panel applet
 * remark: it gets called from a rundll process, wich is different process than tapisrv. 
 * Take into account when debbugging.
 *	\see http://msdn2.microsoft.com/en-us/library/ms725981.aspx
 */
LONG TSPIAPI TUISPI_providerConfig(
    TUISPIDLLCALLBACK pfnUIDLLCallback,
    HWND hwndOwner,
    DWORD dwPermanentProviderID)
{
	DialogBox(g_hinst,
        MAKEINTRESOURCE(IDD_DIALOG),
        hwndOwner,
        (DLGPROC)ConfigDlgProc);

	return (0);
}

//////////////////////////////////////////////////////////////
//
//	TSPI Line Device functions
//
//////////////////////////////////////////////////////////////
LONG
TSPIAPI
TSPI_lineAccept(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    LPCSTR              lpsUserUserInfo,
    DWORD               dwSize
    )
{
	g_xLogger.LogTrace("TSP", "TSPI_lineAccept hdCall="+ToString(hdCall));
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineAddToConference(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdConfCall,
    HDRVCALL            hdConsultCall
    )
{
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineAnswer(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    LPCSTR              lpsUserUserInfo,
    DWORD               dwSize
    )
{
	g_xLogger.LogTrace("TSP", "TSPI_lineAnswer hdCall="+ToString(hdCall));
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineBlindTransfer(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    LPCWSTR             lpszDestAddress,
    DWORD               dwCountryCode)
{
	return (LINEERR_OPERATIONFAILED);
}



LONG
TSPIAPI
TSPI_lineCompleteCall(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    LPDWORD             lpdwCompletionID,
    DWORD               dwCompletionMode,
    DWORD               dwMessageID
    )
{

	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineCompleteTransfer(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    HDRVCALL            hdConsultCall,
    HTAPICALL           htConfCall,
    LPHDRVCALL          lphdConfCall,
    DWORD               dwTransferMode
    )
{
	g_xLogger.LogTrace("TSP", "TSPI_lineSetupTransfer hdCall="+ToString(hdCall));
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineConditionalMediaDetection(
    HDRVLINE            hdLine,
    DWORD               dwMediaModes,
    LPLINECALLPARAMS    const lpCallParams
    )
{
	return (0);
}

LONG
TSPIAPI
TSPI_lineDevSpecific(
    DRV_REQUESTID       dwRequestID,
    HDRVLINE            hdLine,
    DWORD               dwAddressID,
    HDRVCALL            hdCall,
    LPVOID              lpParams,
    DWORD               dwSize
    )
{
	g_xLogger.LogTrace("TSP", "TSPI_lineDevSpecific hdLine="+ToString(hdLine)+", hdCall="+ToString(hdCall));
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineDevSpecificFeature(
    DRV_REQUESTID       dwRequestID,
    HDRVLINE            hdLine,
    DWORD               dwFeature,
    LPVOID              lpParams,
    DWORD               dwSize
    )
{
	g_xLogger.LogTrace("TSP", "TSPI_lineDevSpecificFeature hdLine="+ToString(hdLine)+", feature="
		+ToString(dwFeature));
	return (LINEERR_OPERATIONFAILED);
}

/**
*	This function dials the specified dialable number on the specified call.
*	\see http://msdn2.microsoft.com/en-us/library/ms725542.aspx
*	\param dwRequestID
*	\param hdCall
*	\param lpszDestAddress
*	\param dwCountryCode
*/
LONG
TSPIAPI
TSPI_lineDial(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    LPCWSTR             lpszDestAddress,
    DWORD               dwCountryCode
    )
{
	if (g_xTapiProvider==NULL) {
		return LINEERR_NOMEM ;
	}

	return g_xTapiProvider->TSPI_lineDial(dwRequestID,hdCall,lpszDestAddress,dwCountryCode);
}


LONG
TSPIAPI
TSPI_lineDropOnClose(                                           // TSPI v1.4
    HDRVCALL            hdCall
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineDropNoOwner(                                           // TSPI v1.4
    HDRVCALL            hdCall
    )
{
	return (LINEERR_OPERATIONFAILED);
}


/**
 * lineForward mainly used to simulate do-not-disturb=yes/no
 * \see http://msdn.microsoft.com/library/default.asp?url=/library/en-us/tapi/tspi/tspi_lineforward.asp?frame=true
 *	\param dwRequestId
 *	\param hdLine
 *	\param bAllAddresses
 *	\param lpForwardList
 *	\param dwNumRingsNoAnswer
 *	\param htConsultCall
 *	\param lphdConsultCall
 *	\param lpCallParams
 */
LONG
TSPIAPI
TSPI_lineForward(
    DRV_REQUESTID       dwRequestID,
    HDRVLINE            hdLine,
    DWORD               bAllAddresses,
    DWORD               dwAddressID,
    LPLINEFORWARDLIST   const lpForwardList,
    DWORD               dwNumRingsNoAnswer,
    HTAPICALL           htConsultCall,
    LPHDRVCALL          lphdConsultCall,
    LPLINECALLPARAMS    const lpCallParams
    )
{
	if (g_xTapiProvider==NULL) {
		return LINEERR_NOMEM ;
	}

	return g_xTapiProvider->TSPI_lineForward( dwRequestID, hdLine, bAllAddresses, dwAddressID, lpForwardList, dwNumRingsNoAnswer, htConsultCall, lphdConsultCall, lpCallParams );
}


LONG
TSPIAPI
TSPI_lineGatherDigits(
    HDRVCALL            hdCall,
    DWORD               dwEndToEndID,
    DWORD               dwDigitModes,
    LPWSTR              lpsDigits,
    DWORD               dwNumDigits,
    LPCWSTR             lpszTerminationDigits,
    DWORD               dwFirstDigitTimeout,
    DWORD               dwInterDigitTimeout
    )
{
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineGenerateDigits(
    HDRVCALL            hdCall,
    DWORD               dwEndToEndID,
    DWORD               dwDigitMode,
    LPCWSTR             lpszDigits,
    DWORD               dwDuration
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineGenerateTone(
    HDRVCALL            hdCall,
    DWORD               dwEndToEndID,
    DWORD               dwToneMode,
    DWORD               dwDuration,
    DWORD               dwNumTones,
    LPLINEGENERATETONE  const lpTones
    )
{
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineGetAddressID(
    HDRVLINE            hdLine,
    LPDWORD             lpdwAddressID,
    DWORD               dwAddressMode,
    LPCWSTR             lpsAddress,
    DWORD               dwSize
    )
{
	return (0);
}

LONG
TSPIAPI
TSPI_lineGetAddressStatus(
    HDRVLINE            hdLine,
    DWORD               dwAddressID,
    LPLINEADDRESSSTATUS lpAddressStatus
    )
{
	g_xLogger.LogTrace("TSP", "TSPI_lineGetAddressStatus hdLine="+ToString(hdLine));
	return (0);
}

LONG
TSPIAPI
TSPI_lineGetDevConfig(
    DWORD               dwDeviceID,
    LPVARSTRING         lpDeviceConfig,
    LPCWSTR             lpszDeviceClass
    )
{
	return (0);
}


LONG
TSPIAPI
TSPI_lineGetExtensionID(
    DWORD               dwDeviceID,
    DWORD               dwTSPIVersion,
    LPLINEEXTENSIONID   lpExtensionID
    )
{
	return (0);
}


LONG
TSPIAPI
TSPI_lineGetIcon(
    DWORD               dwDeviceID,
    LPCWSTR             lpszDeviceClass,
    LPHICON             lphIcon
    )
{
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineGetID(
    HDRVLINE            hdLine,
    DWORD               dwAddressID,
    HDRVCALL            hdCall,
    DWORD               dwSelect,
    LPVARSTRING         lpDeviceID,
    LPCWSTR             lpszDeviceClass,
    HANDLE              hTargetProcess                          // TSPI v2.0
    )
{
	return (0);
}



LONG
TSPIAPI
TSPI_lineGetNumAddressIDs(
    HDRVLINE            hdLine,
    LPDWORD             lpdwNumAddressIDs
    )
{
	g_xLogger.LogTrace("TSP", "TSPI_lineGetNumAddressIDs hdLine="+ToString(hdLine));
	// me model a line as a single asterisk extension
	*lpdwNumAddressIDs = 1;

	return (0);
}

/**
*	Function TSPI_lineHold
*	This function is called by TAPI to hold a call
*/
LONG
TSPIAPI
TSPI_lineHold(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall
    )
{
	if (g_xTapiProvider==NULL) {
		return LINEERR_NOMEM ;
	}

	g_xLogger.LogTrace("TSP", "TSPI_lineHold hdCall="+ToString(hdCall));

	//return g_xTapiProvider->TSPI_lineHold(dwRequestID,hdCall);
	return (LINEERR_OPERATIONFAILED);
}



LONG
TSPIAPI
TSPI_lineMonitorDigits(
    HDRVCALL            hdCall,
    DWORD               dwDigitModes
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineMonitorMedia(
    HDRVCALL            hdCall,
    DWORD               dwMediaModes
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineMonitorTones(
    HDRVCALL            hdCall,
    DWORD               dwToneListID,
    LPLINEMONITORTONE   const lpToneList,
    DWORD               dwNumEntries
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineNegotiateExtVersion(
    DWORD               dwDeviceID,
    DWORD               dwTSPIVersion,
    DWORD               dwLowVersion,
    DWORD               dwHighVersion,
    LPDWORD             lpdwExtVersion
    )
{
	return (LINEERR_OPERATIONFAILED);
}



LONG
TSPIAPI
TSPI_linePark(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    DWORD               dwParkMode,
    LPCWSTR             lpszDirAddress,
    LPVARSTRING         lpNonDirAddress
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_linePickup(
    DRV_REQUESTID       dwRequestID,
    HDRVLINE            hdLine,
    DWORD               dwAddressID,
    HTAPICALL           htCall,
    LPHDRVCALL          lphdCall,
    LPCWSTR             lpszDestAddress,
    LPCWSTR             lpszGroupID
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_linePrepareAddToConference(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdConfCall,
    HTAPICALL           htConsultCall,
    LPHDRVCALL          lphdConsultCall,
    LPLINECALLPARAMS    const lpCallParams
    )
{
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineRedirect(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    LPCWSTR             lpszDestAddress,
    DWORD               dwCountryCode
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineReleaseUserUserInfo(                                   // TSPI v1.4
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineRemoveFromConference(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineSecureCall(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineSelectExtVersion(
    HDRVLINE            hdLine,
    DWORD               dwExtVersion
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineSendUserUserInfo(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    LPCSTR              lpsUserUserInfo,
    DWORD               dwSize
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineSetAppSpecific(
    HDRVCALL            hdCall,
    DWORD               dwAppSpecific
    )
{
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineSetCallData(                                           // TSPI v2.0
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    LPVOID              lpCallData,
    DWORD               dwSize
    )
{
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineSetCallParams(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    DWORD               dwBearerMode,
    DWORD               dwMinRate,
    DWORD               dwMaxRate,
    LPLINEDIALPARAMS    const lpDialParams
    )
{
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineSetCallQualityOfService(                               // TSPI v2.0
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    LPVOID              lpSendingFlowspec,
    DWORD               dwSendingFlowspecSize,
    LPVOID              lpReceivingFlowspec,
    DWORD               dwReceivingFlowspecSize
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineSetCallTreatment(                                      // TSPI v2.0
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    DWORD               dwTreatment
    )
{
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineSetCurrentLocation(                                    // TSPI v1.4
    DWORD               dwLocation
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineSetDefaultMediaDetection(
    HDRVLINE            hdLine,
    DWORD               dwMediaModes
    )
{
	return (0);
}


LONG
TSPIAPI
TSPI_lineSetDevConfig(
    DWORD               dwDeviceID,
    LPVOID              const lpDeviceConfig,
    DWORD               dwSize,
    LPCWSTR             lpszDeviceClass
    )
{
	return (LINEERR_OPERATIONFAILED);
}



LONG
TSPIAPI
TSPI_lineSetLineDevStatus(                                      // TSPI v2.0
    DRV_REQUESTID       dwRequestID,
    HDRVLINE            hdLine,
    DWORD               dwStatusToChange,
    DWORD               fStatus
    )
{
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineSetMediaControl(
    HDRVLINE                    hdLine,
    DWORD                       dwAddressID,
    HDRVCALL                    hdCall,
    DWORD                       dwSelect,
    LPLINEMEDIACONTROLDIGIT     const lpDigitList,
    DWORD                       dwDigitNumEntries,
    LPLINEMEDIACONTROLMEDIA     const lpMediaList,
    DWORD                       dwMediaNumEntries,
    LPLINEMEDIACONTROLTONE      const lpToneList,
    DWORD                       dwToneNumEntries,
    LPLINEMEDIACONTROLCALLSTATE const lpCallStateList,
    DWORD                       dwCallStateNumEntries
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineSetMediaMode(
    HDRVCALL            hdCall,
    DWORD               dwMediaMode
    )
{
	return (LINEERR_OPERATIONFAILED);
}

/**
 * Outlook2000 compatibility issue:
 * \see http://groups.google.es/group/microsoft.public.platformsdk.telephony.tsp/browse_frm/thread/d9795ffb3d007e3/1abc870bdfc64569?lnk=st&q=TSP+with+Outlook&rnum=17&hl=es#1abc870bdfc64569
 */
LONG
TSPIAPI
TSPI_lineSetStatusMessages(
    HDRVLINE            hdLine,
    DWORD               dwLineStates,
    DWORD               dwAddressStates
    )
{
	return (0);
}

LONG
TSPIAPI
TSPI_lineSetTerminal(
    DRV_REQUESTID       dwRequestID,
    HDRVLINE            hdLine,
    DWORD               dwAddressID,
    HDRVCALL            hdCall,
    DWORD               dwSelect,
    DWORD               dwTerminalModes,
    DWORD               dwTerminalID,
    DWORD               bEnable
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineSetupConference(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    HDRVLINE            hdLine,
    HTAPICALL           htConfCall,
    LPHDRVCALL          lphdConfCall,
    HTAPICALL           htConsultCall,
    LPHDRVCALL          lphdConsultCall,
    DWORD               dwNumParties,
    LPLINECALLPARAMS    const lpCallParams
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineSetupTransfer(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall,
    HTAPICALL           htConsultCall,
    LPHDRVCALL          lphdConsultCall,
    LPLINECALLPARAMS    const lpCallParams
    )
{
	if (g_xTapiProvider==NULL) {
		return LINEERR_NOMEM ;
	}

	return g_xTapiProvider->TSPI_lineSetupTransfer(dwRequestID,hdCall,htConsultCall,lphdConsultCall,lpCallParams);
}

LONG
TSPIAPI
TSPI_lineSwapHold(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdActiveCall,
    HDRVCALL            hdHeldCall
    )
{
	return (LINEERR_OPERATIONFAILED);
}

LONG
TSPIAPI
TSPI_lineUncompleteCall(
    DRV_REQUESTID       dwRequestID,
    HDRVLINE            hdLine,
    DWORD               dwCompletionID
    )
{
	return (LINEERR_OPERATIONFAILED);
}

/**
*	Function TSPI_lineUnHold
*	This function is called by TAPI to retreive a call
*/

LONG
TSPIAPI
TSPI_lineUnhold(
    DRV_REQUESTID       dwRequestID,
    HDRVCALL            hdCall
    )
{
	if (g_xTapiProvider==NULL) {
		return LINEERR_NOMEM ;
	}
	g_xLogger.LogTrace("TSP", "TSPI_lineUnHold hdCall="+ToString(hdCall));
	//return g_xTapiProvider->TSPI_lineUnHold(dwRequestID,hdCall);
	return (LINEERR_OPERATIONFAILED);
}


LONG
TSPIAPI
TSPI_lineUnpark(
    DRV_REQUESTID       dwRequestID,
    HDRVLINE            hdLine,
    DWORD               dwAddressID,
    HTAPICALL           htCall,
    LPHDRVCALL          lphdCall,
    LPCWSTR             lpszDestAddress
    )
{
	return (LINEERR_OPERATIONFAILED);
}

///////////////////////////////////////////
//
//	TSPI phone Device functions 
//
//////////////////////////////////////////
LONG
TSPIAPI
TSPI_phoneClose(
    HDRVPHONE           hdPhone
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneDevSpecific(
    DRV_REQUESTID       dwRequestID,
    HDRVPHONE           hdPhone,
    LPVOID              lpParams,
    DWORD               dwSize
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneGetButtonInfo(
    HDRVPHONE           hdPhone,
    DWORD               dwButtonLampID,
    LPPHONEBUTTONINFO   lpButtonInfo
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneGetData(
    HDRVPHONE           hdPhone,
    DWORD               dwDataID,
    LPVOID              lpData,
    DWORD               dwSize
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneGetDevCaps(
    DWORD               dwDeviceID,
    DWORD               dwTSPIVersion,
    DWORD               dwExtVersion,
    LPPHONECAPS         lpPhoneCaps
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneGetDisplay(
    HDRVPHONE           hdPhone,
    LPVARSTRING         lpDisplay
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneGetExtensionID(
    DWORD               dwDeviceID,
    DWORD               dwTSPIVersion,
    LPPHONEEXTENSIONID  lpExtensionID
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneGetGain(
    HDRVPHONE           hdPhone,
    DWORD               dwHookSwitchDev,
    LPDWORD             lpdwGain
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneGetHookSwitch(
    HDRVPHONE           hdPhone,
    LPDWORD             lpdwHookSwitchDevs
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}


LONG
TSPIAPI
TSPI_phoneGetIcon(
    DWORD               dwDeviceID,
    LPCWSTR             lpszDeviceClass,
    LPHICON             lphIcon
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}



LONG
TSPIAPI
TSPI_phoneGetID(
    HDRVPHONE           hdPhone,
    LPVARSTRING         lpDeviceID,
    LPCWSTR             lpszDeviceClass,
    HANDLE              hTargetProcess                          // TSPI v2.0
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneGetLamp(
    HDRVPHONE           hdPhone,
    DWORD               dwButtonLampID,
    LPDWORD             lpdwLampMode
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneGetRing(
    HDRVPHONE           hdPhone,
    LPDWORD             lpdwRingMode,
    LPDWORD             lpdwVolume
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneGetStatus(
    HDRVPHONE           hdPhone,
    LPPHONESTATUS       lpPhoneStatus
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneGetVolume(
    HDRVPHONE           hdPhone,
    DWORD               dwHookSwitchDev,
    LPDWORD             lpdwVolume
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneNegotiateExtVersion(
    DWORD               dwDeviceID,
    DWORD               dwTSPIVersion,
    DWORD               dwLowVersion,
    DWORD               dwHighVersion,
    LPDWORD             lpdwExtVersion
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneNegotiateTSPIVersion(
    DWORD               dwDeviceID,
    DWORD               dwLowVersion,
    DWORD               dwHighVersion,
    LPDWORD             lpdwTSPIVersion
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneOpen(
    DWORD               dwDeviceID,
    HTAPIPHONE          htPhone,
    LPHDRVPHONE         lphdPhone,
    DWORD               dwTSPIVersion,
    PHONEEVENT          lpfnEventProc
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneSelectExtVersion(
    HDRVPHONE           hdPhone,
    DWORD               dwExtVersion
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneSetButtonInfo(
    DRV_REQUESTID       dwRequestID,
    HDRVPHONE           hdPhone,
    DWORD               dwButtonLampID,
    LPPHONEBUTTONINFO   const lpButtonInfo
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneSetData(
    DRV_REQUESTID       dwRequestID,
    HDRVPHONE           hdPhone,
    DWORD               dwDataID,
    LPVOID              const lpData,
    DWORD               dwSize
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}


LONG
TSPIAPI
TSPI_phoneSetDisplay(
    DRV_REQUESTID       dwRequestID,
    HDRVPHONE           hdPhone,
    DWORD               dwRow,
    DWORD               dwColumn,
    LPCWSTR             lpsDisplay,
    DWORD               dwSize
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneSetGain(
    DRV_REQUESTID       dwRequestID,
    HDRVPHONE           hdPhone,
    DWORD               dwHookSwitchDev,
    DWORD               dwGain
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneSetHookSwitch(
    DRV_REQUESTID       dwRequestID,
    HDRVPHONE           hdPhone,
    DWORD               dwHookSwitchDevs,
    DWORD               dwHookSwitchMode
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneSetLamp(
    DRV_REQUESTID       dwRequestID,
    HDRVPHONE           hdPhone,
    DWORD               dwButtonLampID,
    DWORD               dwLampMode
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneSetRing(
    DRV_REQUESTID       dwRequestID,
    HDRVPHONE           hdPhone,
    DWORD               dwRingMode,
    DWORD               dwVolume
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneSetStatusMessages(
    HDRVPHONE           hdPhone,
    DWORD               dwPhoneStates,
    DWORD               dwButtonModes,
    DWORD               dwButtonStates
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}

LONG
TSPIAPI
TSPI_phoneSetVolume(
    DRV_REQUESTID       dwRequestID,
    HDRVPHONE           hdPhone,
    DWORD               dwHookSwitchDev,
    DWORD               dwVolume
    )
{
	return (PHONEERR_OPERATIONUNAVAIL);
}



LONG
TSPIAPI
TSPI_providerConfig(
    HWND                hwndOwner,
    DWORD               dwPermanentProviderID
    )
{   //Tapi version 1.4 and earlier - now can be ignored.
	return (0);
}

LONG
TSPIAPI
TSPI_providerCreateLineDevice(                                  // TSPI v1.4
    DWORD_PTR           dwTempID,
    DWORD               dwDeviceID
    )
{
	return (0);
}

LONG
TSPIAPI
TSPI_providerCreatePhoneDevice(                                 // TSPI v1.4
    DWORD_PTR           dwTempID,
    DWORD               dwDeviceID
    )
{
	return (0);
}



LONG
TSPIAPI
TSPI_providerFreeDialogInstance(                                // TSPI v2.0
    HDRVDIALOGINSTANCE  hdDlgInst
    )
{
	return (0);
}


LONG
TSPIAPI
TUISPI_lineConfigDialogEdit(                                    // TSPI v2.0
    TUISPIDLLCALLBACK   lpfnUIDLLCallback,
    DWORD               dwDeviceID,
    HWND                hwndOwner,
    LPCWSTR             lpszDeviceClass,
    LPVOID              const lpDeviceConfigIn,
    DWORD               dwSize,
    LPVARSTRING         lpDeviceConfigOut
    )
{
	return (0);
}

LONG
TSPIAPI
TUISPI_phoneConfigDialog(                                       // TSPI v2.0
    TUISPIDLLCALLBACK   lpfnUIDLLCallback,
    DWORD               dwDeviceID,
    HWND                hwndOwner,
    LPCWSTR             lpszDeviceClass
    )
{
	return (0);
}


LONG
TSPIAPI
TUISPI_providerGenericDialog(                                   // TSPI v2.0
    TUISPIDLLCALLBACK   lpfnUIDLLCallback,
    HTAPIDIALOGINSTANCE htDlgInst,
    LPVOID              lpParams,
    DWORD               dwSize,
    HANDLE              hEvent
    )
{
	return (0);
}

LONG
TSPIAPI
TUISPI_providerGenericDialogData(                               // TSPI v2.0
    HTAPIDIALOGINSTANCE htDlgInst,
    LPVOID              lpParams,
    DWORD               dwSize
    )
{
	return (0);
}
