/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Portions of the code: 
 * Original Copyright  1994-2004 JulMar Entertainment Technology, Inc. All rights reserved.
 *
 * "This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; version 2 of the License.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
 * Public License for more details.
 *
 * adapted to activa 2006 
 *
 */

#include <windows.h>
#include <tapi.h>
#include <string>


/**
*	some helper functions
*/
std::string	wstr2string(LPCWSTR pWstr)
{
	char	charString[1024];
	mbstate_t mbs;
	LONG tr = 0;
    
    mbsinit(&mbs);
	size_t pReturnValue = 0;
	errno_t err = wcsrtombs_s(&pReturnValue, &charString[0],1024, &pWstr, 100, &mbs);
	std::string	str(&charString[0]);
	return str;
}


/**
 * AllocMem 
 */
LPVOID AllocMem (DWORD dwSize)
{   
    LPSTR lpszBuff = NULL;

	// Wrap in exception handler in case the C++ library
	// ever actually follows the specification; it should throw
	// on failure - not return NULL.
    try
    {                 
        lpszBuff = new char [dwSize];
    }
    catch (...)
    {
    }

    return lpszBuff;

}// AllocMem

/**
 * FreeMem
 */
void FreeMem (LPVOID lpBuff)
{
	try
	{
		delete [] lpBuff;
	}
	catch (...)
	{
	}

}// FreeMem


/**
 * CopyBuffer
 */
void CopyBuffer (LPVOID lpDest, LPCVOID lpSource, DWORD dwSize)
{
    memcpy (lpDest, lpSource, (size_t) dwSize);
}// CopyBuffer


/**
 * AddDataBlock
 *	Public function to adjust a variable length structure field for 
 * the library.  This may be called multiple times to add the same 
 * information into a structure.  The offset will only be adjusted if
 * the information in the record is zero.
 */
BOOL AddDataBlock (LPVOID lpVB, DWORD& dwOffset, DWORD& dwSize, 
				   LPCVOID lpBuffer, DWORD dwBufferSize)
{
	// If the buffer size is zero, ignore this.
	if (dwBufferSize == 0)
		return FALSE;
	
	// Cast our block to something which has the first three fields 
	// in the same order as all our variable structures defined by TAPI.
	LPVARSTRING lpVarBuffer = (LPVARSTRING) lpVB;

	DWORD dwUsedSize = lpVarBuffer->dwUsedSize;
#ifndef _X86_
	// Make sure the used size field is DWORD-aligned.  This is required
	// for most non-Intel machines.
	while ((dwUsedSize % 4) != 0)
	{
		dwUsedSize++;
		lpVarBuffer->dwNeededSize++;
	}
#endif

	// If we don't have enough room for the buffer here then exit.
	if (lpVarBuffer->dwTotalSize < dwUsedSize+dwBufferSize)
		return FALSE;

	// We have enough room, save the new used size.
	lpVarBuffer->dwUsedSize = dwUsedSize;

	// Fill in the buffer.
	CopyBuffer ((LPSTR)lpVarBuffer+lpVarBuffer->dwUsedSize, lpBuffer, dwBufferSize);

	// Adjust the offset and size.  We only adjust the offset if it is zero
	// since we may be APPENDING data to an existing block.
	if (dwOffset == 0)
		dwOffset = lpVarBuffer->dwUsedSize;
	dwSize += dwBufferSize;
	lpVarBuffer->dwUsedSize += dwBufferSize;

	// Added the block successfully.
	return TRUE;
	
}// AddDataBlock

/**
*	Public function to add a string to a VARSTRING type buffer.
*/
BOOL AddDataBlock (LPVOID lpVB, DWORD& dwOffset, DWORD& dwSize, LPCSTR lpszBuff)
{
	// Convert the ANSI to UNICODE
	BOOL fRC = FALSE;
	int iSize = MultiByteToWideChar(CP_ACP, 0, lpszBuff, -1, NULL, 0);
	if (iSize > 0)
	{
		// Adjust the NEEDED size for UNICODE strings when the passed
		// data was a normal string (and therefore calculated incorrectly).
		LPVARSTRING lpVarBuffer = (LPVARSTRING) lpVB;
		lpVarBuffer->dwNeededSize += (iSize*sizeof(wchar_t));

		// Convert the buffer
		LPWSTR lpMyString = (LPWSTR) AllocMem (iSize*sizeof(wchar_t));
		if (lpMyString != NULL)
		{
			MultiByteToWideChar(CP_ACP, 0, lpszBuff, -1, lpMyString, iSize);
			fRC = AddDataBlock (lpVB, dwOffset, dwSize, lpMyString, iSize*sizeof(wchar_t));
			FreeMem (lpMyString);
		}
	}
	return fRC;

}// AddDataBlock
