/*
	Name: config.cpp
	Copyright: Under the Mozilla Public License Version 1.1 or later
	Author: ACTIVA
	Date: MAY/06
*/

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Activa ActivaTSP.
 *
 * The Initial Developer of the Original Code is ACTIVA.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): none
 *
 * ***** END LICENSE BLOCK ***** */

#include <windows.h>
#include <tchar.h> 

#include "config.h"
#include "../astprovider/src/ASTLogger.h"
#include "../ActivaAPI/ActivaAPI.h"	// SVN_REVISION

const char HK_ACTIVATSP[] = "Software\\ThinkroSystem_CallBoxTSP";
const HKEY hKeyLM = HKEY_LOCAL_MACHINE;

const char CONFIG_HOST_IP[] = "host";
const char CONFIG_HOST_PORT[] = "port";
const char CONFIG_LOGIN_USER[] = "user";
const char CONFIG_LOGIN_PWD[] = "password";
const char CONFIG_LOGIN_AUTHTYPE[] = "authType";
const char CONFIG_DEVICE_NAME_PREFIX[] = "dnPrefix";
const char CONFIG_DEVICE_MAX[] = "dnNumDevices";
const char CONFIG_DEVICE_DN_[] = "dn";
const char CONFIG_DEVICE_CALLER_ID_NAME_[] = "calleridname";
const char CONFIG_OUTGOING_CONTEXT[] = "outgoingContext";
const char CONFIG_NUM_LINES[] = "numLines";
const char CONFIG_LOGTOFILE[] = "logToFile";
const char CONFIG_AGENT_CONTEXT[] = "agentContext";
const char CONFIG_LOGLEVEL[] = "logLevel";
const char CONFIG_LOG_FILE_PATH[] = "logFilePath";
const char CONFIG_MAXLOGFILES[] = "maxLogFiles";
const char CONFIG_LOGFILESIZE[] = "logFileSize";
const char CONFIG_MAXBUFFERSIZE[] = "MaxBufferSize";
const char CONFIG_ORIGINATE_VARIABLE[] = "OriginateVariable";

std::string	registryGetHostIp(std::string defaultValue) 
{
	std::string	str = "";
	if (!registryGet(CONFIG_HOST_IP, str )) {
		return "localhost";
	}
	return str;
}

DWORD	registryGetHostPort(DWORD defaultValue) 
{
	DWORD	num=0;
	if (!registryGet(CONFIG_HOST_PORT, num )) {
		return 3010;
	}
	return num;
}

std::string	registryGetLoginUser(std::string defaultValue) 
{
	std::string	str = "";
	if (!registryGet(CONFIG_LOGIN_USER, str )) {
		return "mark";
	}
	return str;
}

std::string	registryGetLoginPassword(std::string defaultValue) 
{
	std::string	str = "";
	if (!registryGet(CONFIG_LOGIN_PWD, str )) {
		return "nosecrets";
	}
	return str;
}

DWORD	registryGetLoginAuthType(DWORD defaultValue) 
{
	DWORD	num=0;
	if (!registryGet(CONFIG_LOGIN_AUTHTYPE, num )) {
		return 0;
	}
	return num;
}

std::string	registryGetDnNamePrefix(std::string defaultValue) 
{
	std::string	str = "";
	if(!registryGet(CONFIG_DEVICE_NAME_PREFIX, str )) {
		return "";
	}
	return str;
}

DWORD	registryGetNumLines(DWORD defaultValue) 
{
	DWORD	num=0;
	if (!registryGet(CONFIG_NUM_LINES, num )) {
		return 1;
	}
	return num;
}

// can read multiple indexes. 
std::string	registryGetDn(int index) 
{
	return "callbox";
}

// can read multiple indexes. 
std::string	registryGetCallerIdName(int index) 
{
	std::string	str = "callbox";
	return str;
}

std::string	registryGetOutgoingContext(std::string defaultValue) 
{
	std::string	str = "local";
	return str;
}


std::string	registryGetAgentContext(std::string defaultValue)
{
	std::string str = "";
	if (!registryGet(CONFIG_AGENT_CONTEXT, str)){
		return defaultValue;
	}
	return str;
}

std::string	registryGetOriginateVariable(std::string defaultValue) 
{
	std::string	str = "";
	if (!registryGet(CONFIG_ORIGINATE_VARIABLE, str )) {
		return defaultValue;
	}
	return str;
}


void		registrySetHostIp(std::string value) 
{
	registrySet(CONFIG_HOST_IP, value );
}

void		registrySetHostPort(DWORD value) 
{
	registrySet(CONFIG_HOST_PORT, value );
};


std::string	registryGetLogFilePath(std::string defaultValue) 
{
	return "";
}

bool		registryIsTraceToFile(bool defaultValue) 
{
	return false;
};

int		registryGetLogLevel(int defaultValue) 
{
	DWORD	dwLogToFile = defaultValue;
	if (registryGet(CONFIG_LOGLEVEL, dwLogToFile)) {
		return dwLogToFile;
	}

	return defaultValue;
};

int		registryGetMaxLogFiles(int defaultValue) 
{
	DWORD	dwMaxLogFiles = defaultValue;
	if (registryGet(CONFIG_MAXLOGFILES, dwMaxLogFiles)) {
		return dwMaxLogFiles;
	}

	return defaultValue;
};

int		registryGetLogFileSize(int defaultValue) 
{
	DWORD	dwLogFileSize = defaultValue;
	if (registryGet(CONFIG_LOGFILESIZE, dwLogFileSize)) {
		return dwLogFileSize;
	}

	return defaultValue;
};

int		registryGetMaxBufferSize(int defaultValue)
{
	DWORD	dwMaxBufferSize = defaultValue;
	if (registryGet(CONFIG_MAXBUFFERSIZE, dwMaxBufferSize)) {
		return dwMaxBufferSize;
	}
	return defaultValue;
}

bool registryInit(void)
{
	HKEY	hSwKey;
	DWORD	result;

	if ( RegOpenKeyEx(hKeyLM, "Software", 0, KEY_ALL_ACCESS , &hSwKey) == ERROR_SUCCESS ) {
		HKEY hActivaTspKey;
		RegCreateKeyEx( hSwKey, "ThinkroSystem_CallBoxTSP", 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hActivaTspKey, &result );

		RegCloseKey(hActivaTspKey);
		RegCloseKey(hSwKey);

		return true;
	}

	return false;
}

bool registrySet(std::string key, std::string value)
{
	HKEY hKey;

	if ( RegOpenKeyEx(hKeyLM, HK_ACTIVATSP, 0, KEY_ALL_ACCESS, &hKey) == ERROR_SUCCESS ) {
		if ( RegSetValueEx(hKey, key.c_str(), 0, REG_SZ,(const BYTE *) value.c_str(), (DWORD) value.size()) != ERROR_SUCCESS ) {
			return false;
		}
		RegCloseKey(hKey);
	}
	else
	{
		return false;
	}
	return true;
}

bool registryGet(std::string key, std::string &value)
{
	HKEY hKey;
	char tempStr[256];
	DWORD type;
	DWORD length = sizeof(tempStr);

	value = "";

	if ( RegOpenKeyEx(hKeyLM, HK_ACTIVATSP, 0, KEY_READ, &hKey) == ERROR_SUCCESS) {
		if ( RegQueryValueEx(hKey, key.c_str(), 0, &type,(BYTE*) &tempStr[0], &length ) != ERROR_SUCCESS ) {
			return false;
		}
		else if ( REG_SZ != type ) {
			return false;
		}
		RegCloseKey(hKey);
	}
	else {
		// tdb: trace
		return false;
	}
	tempStr[length] = 0;
	value = tempStr;
	return true;
}


bool registrySet(std::string key, DWORD value)
{
	HKEY hKey;

	if ( RegOpenKeyEx(hKeyLM, HK_ACTIVATSP, 0, KEY_ALL_ACCESS, &hKey) == ERROR_SUCCESS ) {
		if ( RegSetValueEx(hKey, key.c_str(), 0, REG_DWORD, (const BYTE *)&value, sizeof(DWORD)) != ERROR_SUCCESS ) {
			return false;
		}
		RegCloseKey(hKey);
	}
	else {
		return false;
	}
	return true;
}

bool registryGet(std::string key, DWORD &value)
{
	HKEY hKey;
	char tempStr[256];
	DWORD type;
	DWORD length = sizeof(tempStr);

	value = 0;

	if ( RegOpenKeyEx(hKeyLM, HK_ACTIVATSP, 0,KEY_READ, &hKey) == ERROR_SUCCESS ) {
		if ( RegQueryValueEx(hKey, key.c_str(), 0, &type, (BYTE*)&value, &length ) != ERROR_SUCCESS ) {
			return false;
		}
		else if ( REG_DWORD != type ) {
			return false;
		}
		RegCloseKey(hKey);
	}
	else {
		return false;
	}
	return true;

}



/**
 * using GetFileVersionInfo  to retreieve tsp version
 *
 * http://groups.google.es/group/microsoft.public.vc.language/browse_frm/thread/a080da946944c625/5283d0a4c2aa4c77?lnk=st&q=&rnum=1&hl=es#5283d0a4c2aa4c77
 */
std::string	getVersion()
{
		LPTSTR lpszFilePath = "activatsp.tsp"; 
        DWORD dwDummy; 
        
		DWORD dwFVISize = GetFileVersionInfoSize( lpszFilePath , &dwDummy ); 
        
		LPBYTE lpVersionInfo = new BYTE[dwFVISize]; 
        GetFileVersionInfo( lpszFilePath , 0 , dwFVISize , lpVersionInfo ); 
        
		UINT uLen; 
        VS_FIXEDFILEINFO *lpFfi; 
        
		VerQueryValue( lpVersionInfo , _T("\\") , (LPVOID *)&lpFfi , &uLen ); 
        
		DWORD dwFileVersionMS = lpFfi->dwFileVersionMS; 
        DWORD dwFileVersionLS = lpFfi->dwFileVersionLS; 
        
        DWORD dwLeftMost = HIWORD(dwFileVersionMS); 
        DWORD dwSecondLeft = LOWORD(dwFileVersionMS); 
        DWORD dwSecondRight = HIWORD(dwFileVersionLS); 
        DWORD dwRightMost = LOWORD(dwFileVersionLS); 

		delete [] lpVersionInfo; 
        
		std::string	version ;
		version = "V" + ToString(dwLeftMost); 
		version = version +"."+ ToString(dwSecondLeft); 
		version = version +"."+ ToString(dwSecondRight); 
		version = version +"."+ ToString(dwRightMost); 
		version = version +" "+SVN_REVISION;
		return version;
}


/**
 * TSP config dialog
 * invoked through control panel applet
 */
BOOL CALLBACK ConfigDlgProc(
    HWND    hwnd,
    UINT    nMsg,
    WPARAM  wparam,
    LPARAM  lparam)
{
    BOOL    b = FALSE;
	char szTemp[256];
	std::string temp;
	DWORD tempInt;
	std::string	sOutgoingContext, sAgentContext;

    switch( nMsg )
    {
    case WM_INITDIALOG:
		registryInit();

		SetDlgItemText(hwnd, IDC_EDIT_IP, registryGetHostIp("localhost").c_str());
		SetDlgItemInt (hwnd, IDC_EDIT_PORT, registryGetHostPort(3010), FALSE);
		
        b = TRUE;
    break;

    case WM_COMMAND:
        switch( wparam )
        {
		case IDOK:
			
			GetDlgItemText(hwnd, IDC_EDIT_IP, szTemp, sizeof(szTemp));
			registrySetHostIp(szTemp);
			tempInt = GetDlgItemInt(hwnd, IDC_EDIT_PORT, NULL, FALSE);
			registrySetHostPort(tempInt);

			b= TRUE;

        case IDCANCEL:
            EndDialog(hwnd, IDCANCEL);
	        break;
        break;
        }
    break;
    }

    return b;
}
