/*
	Name: astlogger.h
	Copyright: Under the Mozilla Public License Version 1.1 or later
	Author: ACTIVA
	Date: JULY/06
*/

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is astprovider.
 *
 * The Initial Developer of the Original Code is ACTIVA.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): none
 *
 * ***** END LICENSE BLOCK ***** */

#pragma once

#ifndef ASTLOGGER_H
#define ASTLOGGER_H

#include <windows.h>	// LPCSTR
#include <string>
#include "../../ActivaAPI/ActivaAPI.h"

#define	TRACE_LEVEL_ERROR		1	
#define	TRACE_LEVEL_WARNING		2	
#define	TRACE_LEVEL_INFO		4
#define	TRACE_LEVEL_TRACE		8
#define	TRACE_LEVEL_SNIFF		16

class	ASTLogger {
public:
	ASTLogger();
	void	SetLogger(ASTLogListener *logger) { m_logger = logger;};
	void	SetOrigin(std::string sOrigin) {m_origin=sOrigin;};

	void	LogTrace0(LPCSTR pszFormat, ...);
	void	LogTrace(LPCSTR pszFormat, ...);
	void	LogInfo(LPCSTR pszFormat, ...);
	void	LogWarn(LPCSTR pszFormat, ...);
	void	LogError(LPCSTR pszFormat, ...);
protected:
	ASTLogListener	*m_logger;
	std::string		m_origin;
};

/**
 * BasicLogger is just a very simple ASTLogger. It is not intended to be 
 * used in any real app.
 * It is intended to show you how you can derive your own specific logger.
 */
/// BasicLogger is just a very simple logger
class BasicLogger : public ASTLogListener {

private:	
	BasicLogger(); //disallow default constructor
	BasicLogger(const BasicLogger&); //disallow copy construction

public:
	BasicLogger(std::string filename);
	BasicLogger(std::string filename, int maxLogFiles); 
	BasicLogger(std::string filename, int maxLogFiles, int logBackupFileSize, int MaxBufferSize);
	~BasicLogger();
protected:	
	std::string	m_filename;
	std::string m_sMsgBuffer;
	time_t		m_lLastTime;
	int			m_MaxBufferSize;
	bool		m_bEnabled;
	int			m_maxBackupIndex;
	int			m_traceLevel;
	DWORD		m_logBackupFileSize;
	void		trace(std::string sMsg);

	CRITICAL_SECTION	m_csLock;
	CRITICAL_SECTION	m_csDiskLock;

	void	Init(std::string filename, int maxLogFiles, int logBackupFileSize, int MaxBufferSize) ;
	void	Lock() { EnterCriticalSection(&m_csLock); };
	void	Unlock() { LeaveCriticalSection(&m_csLock); };
	void	DiskLock() { EnterCriticalSection(&m_csDiskLock); };
	void	DiskUnlock() { LeaveCriticalSection(&m_csDiskLock); };
	void Rollup() ;
	void WriteBufferToFile();

public:
	void LogTrace0(const std::string & sOrigin, const std::string & sMsg) ;
	void LogTrace(const std::string & sOrigin, const std::string & sMsg) ;
	void LogInfo(const std::string & sOrigin, const std::string & sMsg) ;
	void LogWarn(const std::string & sOrigin, const std::string & sMsg) ;
	void LogError(const std::string & sOrigin, const std::string & sMsg) ;
	void EnableTraceToFile (bool enable) {m_bEnabled=enable;};
	void SetTraceLevel (int level) {m_traceLevel=level;};
	


};

#endif	// ASTLOGGER_H