/*
	Name: astprovider.h
	Copyright: Under the Mozilla Public License Version 1.1 or later
	Author: ACTIVA
	Date: MAY/06
*/

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is astprovider.
 *
 * The Initial Developer of the Original Code is ACTIVA.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): none
 *
 * ***** END LICENSE BLOCK ***** */

#pragma once

#ifndef ASTPROVIDER_H
#define ASTPROVIDER_H

#include <string>
#include <map>
#include <winsock.h>

#include "../../ActivaAPI/ActivaAPI.h"
#include "astinterface.h"
#include "astlogger.h"
#include "invokeidmanager.h"

/// map of {uniqueId, AsteriskCall}
typedef	std::map<std::string, AsteriskCall*>	Map_UniqueId2AsteriskCall ;


/// Encapsulates Asterisk 1.x manager API
class ASTProvider 
{
protected:
	// hide constructor
	ASTProvider(void);

public:
	/// ASTProvider factory method
	static ASTProvider*	Create(ASTListener* listener, ASTLogListener* logger);
	~ASTProvider(void);

	/// connect to an asterisk box
	AC_RESULT	Connect(std::string sHost, int nPort, std::string user, std::string pwd, AUTH_TYPE eAuthType);
	
	/// disconnects from asterisk box
	bool		Disconnect();

	/// bridge two active channels
	AC_RESULT	Bridge(std::string channel1, std::string channel2, bool tone);

	/// originates a call
	AC_RESULT	Originate(std::string channel, std::string callerId, std::string callerIdName, std::string calledExtension, DWORD callid, std::string context, DWORD timeout, std::string variable, bool bLockReaderAfterResponse = false);
	
	/// hangup a given channel
	AC_RESULT	Hangup(std::string channel);

	/// adds an extension interface to a queue
	AC_RESULT	QueueAdd(std::string queue, std::string channelInterface, int penalty, bool paused );

	/// removes an extension interface to a queue
	AC_RESULT	QueueRemove(std::string queue, std::string channelInterface);

	///pauses/upauses an already logged-in agent
	AC_RESULT	QueuePause(std::string channelInterface, bool bPaused);

	/// executes an application via originate AMI
	AC_RESULT	OriginateApplication(std::string channel, std::string application, std::string data, std::string variable, DWORD timeout);

	/// Redirect (transfer) a call
	AC_RESULT	Redirect(std::string channel, std::string extraChannel, std::string extension, std::string context);

	/// Set a global or local channel variable.
	AC_RESULT	SetVar(std::string channel, std::string variable, std::string value);

	/// Get a global or local channel variable.
	AC_RESULT	GetVar(std::string channel, std::string variable, std::string &value);
	
	/// Monitor a call (Recording)
	AC_RESULT	Monitor(std::string channel, std::string file, std::string format, std::string mix);

	/// StopMonitor a call (stop Recording)
	AC_RESULT	StopMonitor(std::string channel);

	/// PauseMonitor a call (pause Recording)
	AC_RESULT	PauseMonitor(std::string channel);

	/// UnpauseMonitor a call (continue Recording after pause)
	AC_RESULT	UnpauseMonitor(std::string channel);

	/// ChangeMonitor a call ( change filename of the Recording after start)
	AC_RESULT	ChangeMonitor(std::string channel, std::string file);

	///	UnLocks Reader Thread
	void		UnLockReader() { m_invokeIdManager.UnLockReader();};

/// test helpers methods
#ifdef	__ACTIVATSP_CPPUNIT_BUILD
	void	TestPL(std::string line) { ParseLine(line);};
#endif

private:
	ASTLogger				m_log;
	ASTListener				*m_listener;
	InvokeIdManager			m_invokeIdManager;
	
	AstManTrans				m_AstManTrans; //Asterisk version translator helper

    SOCKET					m_socket;	//Our connection to the Asterisk box
	HANDLE					m_hMutex;	// protect access
	bool					m_bDie ;	
	bool					m_bProcessingCommandResult;
	HANDLE					m_hReaderThread;

	DWORD	ReaderProc(void);
	bool	SendCommand(std::string command);
	void	LinkDownProcedure() ;

	std::map<std::string, std::string>	m_bag;	// key/value map
	std::string	m_commandResult;

	AC_RESULT	Login(std::string user, std::string pwd, AUTH_TYPE eAuthType);
	AC_RESULT	OriginateByDial(std::string callingDevice, std::string calledDevice, DWORD callid, std::string outGoingChannel, DWORD timeout, std::string invokeId);
	AC_RESULT	OriginateByContext(std::string channel, std::string callerId, std::string callerIdName, std::string calledExtension, DWORD callid, std::string context, DWORD timeout, std::string invokeId, std::string variable);

	bool		ConnectSocket(std::string sHost, int nPort);
	void		SetThreadHandle(HANDLE thHandl) { m_hReaderThread = thHandl; }
	std::string	ReadLine() ;
	void		ParseLine(std::string line);
	void		WaitReaderThread()  ;

	void		HandleResponse(std::map<std::string, std::string> bag);
	void		HandleEvent(std::map<std::string, std::string>	bag);

	// friend
	friend DWORD WINAPI ProviderThreadProc(LPVOID lpParameter);
	
};


DWORD WINAPI ProviderThreadProc(LPVOID lpParameter);


#endif	// ASTPROVIDER_H