/*
	Name: invokeidmanager.h
	Copyright: Under the Mozilla Public License Version 1.1 or later
	Author: ACTIVA
	Date: MAY/06
*/

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is astprovider.
 *
 * The Initial Developer of the Original Code is ACTIVA.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): none
 *
 * ***** END LICENSE BLOCK ***** */

#pragma once

#ifndef INVOKEIDMANAGER_H
#define INVOKEIDMANAGER_H

#include <string>
#include <map>

#include "astinterface.h"


class InvokeID 
{
	std::string	m_invoke;
	bool		m_bReceived ;
	ManagerResponse m_result ;
	HANDLE		m_hEvent;	// signals response reception
	bool		m_bLockReaderAfterResponse;

public:
	InvokeID(std::string invoke, bool bLockReaderAfterResponse = false);
		~InvokeID();

	// waits until response is received or timeout
	bool	WaitReponse(  int seconds ) ;
	
	// signals response received and stores result
	void	SignalResponse(ManagerResponse result) ;
	
	// retrieves result
	ManagerResponse GetResult() ;

	// retrive lock reader condition
	bool	GetFlagLockReaderAfterResponse(){return(m_bLockReaderAfterResponse);};
};


typedef std::map<std::string, InvokeID*> Map_invokeId ;

/*
 * class InvokeIdManager
 */
class InvokeIdManager
{
private:
    int    			m_currentInvokeId ;
	Map_invokeId	m_pendingInvokeID;
	HANDLE			m_hMutex;	// protects map shared access
	HANDLE			m_hManagerEvent; // signals readerThread after completion proc 


public:
		InvokeIdManager() ;

	// waits until response is received or timeout
	bool	WaitResponse(std::string iInvoke, ManagerResponse &result, int timeoutSeconds) ;

	// generates new invokeid and stores it. returns handler
	std::string	NewInvokeId(bool bLockReaderAfterResponse = false);

	// signals response received and stores result
    bool	SignalResponse(ManagerResponse result, std::string invokeID) ;

	// signals all pending invokeid
	int		SignalAll(ManagerResponse result);

	//unlock reader if its locked
	void	UnLockReader();

private:
	ManagerResponse	GetResult(std::string iInvoke) ;
};


#endif	// INVOKEIDMANAGER_H
