/*
	Name: tracer.cpp
	Copyright: Under the Mozilla Public License Version 1.1 or later
	Author: ACTIVA
	Date: JULY/06
*/

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Activa.
 *
 * The Initial Developer of the Original Code is ACTIVA TEAM.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): none
 *
 * ***** END LICENSE BLOCK ***** */

#include "astprovider.h"
#include <time.h>
#include <stdio.h>

#define	DEFAULT_FILE_SIZE	(4*1000*1000)		// 4 MB
#define MIN_DEFAULT_FILE_SIZE (1*1000*1000)		// 1 MB
#define DEFAULT_LOG_FILES	10
#define SIZE 26
#define BUFFER_SIZE (4*1024)
#define MSG_SIZE (BUFFER_SIZE-74)
#define DEFAULT_MAX_BUFFER_SIZE (60*1000)


//
void	ASTLogListener::LogTrace0(const std::string & sOrigin, const std::string & msg)
{
	// do nothing
};

void	ASTLogListener::LogTrace(const std::string & sOrigin, const std::string & msg)
{
	// do nothing
};

void	ASTLogListener::LogInfo(const std::string & sOrigin, const std::string & msg)
{
	// do nothing
};

void	ASTLogListener::LogWarn(const std::string & sOrigin, const std::string & msg)
{
	// do nothing
};

void	ASTLogListener::LogError(const std::string & sOrigin, const std::string & msg)
{
	// do nothing
};



//
ASTLogger::ASTLogger()
{
	m_logger = NULL;
	m_origin = "";
};

void	ASTLogger::LogTrace0(LPCSTR pszFormat, ...)
{
    va_list ap;	
    va_start(ap, pszFormat);
	if (m_logger) {
		char    buffer[4*1024];
		wvsprintf(buffer, pszFormat, ap);
		m_logger->LogTrace0(m_origin, buffer);
	}
    va_end(ap);
};

void	ASTLogger::LogTrace(LPCSTR pszFormat, ...)
{
    va_list ap;	
    va_start(ap, pszFormat);
	if (m_logger) {
		char    buffer[4*1024];
		wvsprintf(buffer, pszFormat, ap);
		m_logger->LogTrace(m_origin, buffer);
	}
    va_end(ap);
};

void	ASTLogger::LogInfo(LPCSTR pszFormat, ...)
{
    va_list ap;	
    va_start(ap, pszFormat);
	if (m_logger) {
		char    buffer[4*1024];
		wvsprintf(buffer, pszFormat, ap);
		m_logger->LogInfo(m_origin, buffer);
	}

	va_end(ap);
};

void	ASTLogger::LogWarn(LPCSTR pszFormat, ...)
{
    va_list ap;	
    va_start(ap, pszFormat);
	if (m_logger) {
		char    buffer[4*1024];
		wvsprintf(buffer, pszFormat, ap);
		m_logger->LogWarn(m_origin, buffer);
	}
    va_end(ap);
};

void	ASTLogger::LogError(LPCSTR pszFormat, ...)
{
    va_list ap;	
    va_start(ap, pszFormat);
	if (m_logger) {
		char    buffer[4*1024];
		wvsprintf(buffer, pszFormat, ap);
		m_logger->LogError(m_origin, buffer);
	}
    va_end(ap);
};

/**
 * BasicLogger is just a very simple Logger. It is not intended to be 
 * used in any real app.
 * It is intended to show you how you can derive your own specific logger.
 */
BasicLogger::BasicLogger(std::string filename, int maxLogFiles, int logBackupFileSize, int MaxBufferSize) 
{
	Init(filename, maxLogFiles, logBackupFileSize, MaxBufferSize);
}

BasicLogger::BasicLogger(std::string filename, int maxLogFiles) 
{
	Init(filename, maxLogFiles, DEFAULT_FILE_SIZE, DEFAULT_MAX_BUFFER_SIZE);
}

BasicLogger::BasicLogger(std::string filename) 
{
	Init(filename, DEFAULT_LOG_FILES, DEFAULT_FILE_SIZE, DEFAULT_MAX_BUFFER_SIZE);
}

void BasicLogger::Init(std::string filename, int maxLogFiles, int logBackupFileSize, int MaxBufferSize) 
{
	m_filename = filename;
	m_maxBackupIndex = maxLogFiles;
	m_sMsgBuffer="";
	m_lLastTime=time(NULL);
	m_MaxBufferSize=MaxBufferSize;

	if(logBackupFileSize>MIN_DEFAULT_FILE_SIZE)
		m_logBackupFileSize = logBackupFileSize;
	else
		m_logBackupFileSize = MIN_DEFAULT_FILE_SIZE;
	
	m_traceLevel = TRACE_LEVEL_ERROR 
					| TRACE_LEVEL_WARNING
					| TRACE_LEVEL_INFO	
					| TRACE_LEVEL_TRACE	
					| TRACE_LEVEL_SNIFF	;

	InitializeCriticalSection(&m_csDiskLock);
	InitializeCriticalSection(&m_csLock);
}


BasicLogger::~BasicLogger()
{
	//On destruction flush the buffer
	WriteBufferToFile();
	DeleteCriticalSection(&m_csLock);
	DeleteCriticalSection(&m_csDiskLock);
}

void BasicLogger::trace(std::string sMsg) 
{
	if (!m_bEnabled) 
		return;

	char _date[SIZE];
	char _time[SIZE];
	char msg[BUFFER_SIZE]="";

	//#2526111 Too large numLines produces an exception
	if(sMsg.size()>MSG_SIZE)
		sMsg = sMsg.substr(0,MSG_SIZE);

	Lock();
	//operating system date
    _strdate_s( _date, SIZE );
	//operating system time
    _strtime_s( _time, SIZE );
	//get current_id and concatenate with message
	sprintf_s( msg, BUFFER_SIZE, "%s %s [%d] %s\r\n", _date, _time, GetCurrentThreadId(), sMsg.c_str());
	
	m_sMsgBuffer.append(msg);

	if (m_sMsgBuffer.size() > m_MaxBufferSize || time(NULL) > m_lLastTime + 1)
	{
		WriteBufferToFile();
	}
	
	Unlock();
};

void BasicLogger::WriteBufferToFile()
{
	DWORD	dwNewFilePointer = 0;
	DWORD   nBytes;
	std::string		tmpTraceBuff;
	const	char*	tmpBuffer;

	Lock();
	//tmpTraceBuff="---------------\n ----------Buffer-----------\n---------------\n"+m_sMsgBuffer+"---------------\n ----------FINBuffer-----------\n---------------\n\n\n";
	
	tmpTraceBuff=m_sMsgBuffer;
	tmpBuffer = tmpTraceBuff.c_str();
	m_sMsgBuffer = "";
	m_lLastTime=time(NULL); 
	
	//do not use DiskLock in other places it will cause a potential Deathlock (never do a DiskLock-Lock sequence)
	//we do this to ensure ordered traces
	DiskLock();
	Unlock();
	
	HANDLE  hFile = CreateFile(m_filename.c_str(), GENERIC_WRITE, 0, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
	dwNewFilePointer = SetFilePointer(hFile, 0, 0, FILE_END);
	WriteFile(hFile, tmpBuffer, (DWORD)strlen(tmpBuffer), &nBytes, 0);
	CloseHandle(hFile);

	if (dwNewFilePointer>m_logBackupFileSize) {
		Rollup();
	}
	DiskUnlock();
	
}

void BasicLogger::Rollup() 
{
	BOOL rc ;

    if (m_maxBackupIndex > 0) {
        for(unsigned int i = m_maxBackupIndex; i > 1; i--) {
            std::string oldName = m_filename ;
				oldName = oldName + ".";
				oldName = oldName + ToString(i-1);
            std::string newName = m_filename ;
				newName = newName + "." ;
				newName = newName + ToString(i);

			rc = MoveFileEx(oldName.c_str(), newName.c_str(), MOVEFILE_REPLACE_EXISTING);
        }
        std::string newName = m_filename + ".1";

		rc = MoveFileEx(m_filename.c_str(), newName.c_str(), MOVEFILE_REPLACE_EXISTING);
    }
}

void BasicLogger::LogTrace0(const std::string & sOrigin, const std::string & sMsg) { 
	if (m_traceLevel & TRACE_LEVEL_SNIFF) {
		trace("TRACE0["+sOrigin+"] "+sMsg);
	}
};

void BasicLogger::LogTrace(const std::string & sOrigin, const std::string & sMsg) { 
	if (m_traceLevel & TRACE_LEVEL_TRACE) {
		trace("TRACE ["+sOrigin+"] "+sMsg);
	}
};

void BasicLogger::LogInfo(const std::string & sOrigin, const std::string & sMsg) { 
	if (m_traceLevel & TRACE_LEVEL_INFO) {
		trace("INFO  ["+sOrigin+"] "+sMsg);
	}
};

void BasicLogger::LogWarn(const std::string & sOrigin, const std::string & sMsg) { 
	if (m_traceLevel & TRACE_LEVEL_WARNING) {
		trace("WARN  ["+sOrigin+"] "+sMsg);
	}
};

void BasicLogger::LogError(const std::string & sOrigin, const std::string & sMsg) { 
	if (m_traceLevel & TRACE_LEVEL_ERROR) {
		trace("ERROR ["+sOrigin+"] "+sMsg);
	}
};

