/*
	Name: astprovidertest.cpp
	Copyright: Under the Mozilla Public License Version 1.1 or later
	Author: ACTIVA
	Date: MAY/06
*/

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is astprovider.
 *
 * The Initial Developer of the Original Code is ACTIVA.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): none
 *
 * ***** END LICENSE BLOCK ***** */

#include "stdafx.h"
#include <time.h>

#include "../ActivaAPI/ActivaAPI.h"
#include "ASTProviderTest.h"
#include "../astprovider/src/astlogger.h"

#define MAX_LOADSTRING 100
#define SIZE 26

#ifdef _DEBUG
#	define ACTIVAAPI_LIBRARY "ActivaAPId.DLL"
#else
#	define ACTIVAAPI_LIBRARY "ActivaAPI.DLL"
#endif

static LPFN_CreateASTCSTAProvider	g_CreateASTCSTAProvider;
static LPFN_DeleteASTCSTAProvider	g_DeleteASTCSTAProvider;
static HINSTANCE					g_hInstActivaAPI;

// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];					// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];			// the main window class name

HWND g_hWnd, g_hWndTraceWindow ;
ASTCstaProviderInterface*	g_Provider = NULL;			// asterisk manager api provider
ASTCstaListener*	g_Listener = NULL;			// astprovider listerenr
MonitorXref			g_xref = (MonitorXref) -1;
std::string			g_dn = "419";
std::string			g_queue = "100";
std::string			g_to = "422";
std::string			g_ip = "asterisk1";
std::string			g_login = "admin";
std::string			g_pwd = "amp111";
std::string			g_calleridname = "Nuria";
std::string			g_callerid = "Nuria <422>";
std::string			g_outbound_context = "from-internal";
std::string			g_incoming_context = "from-internal";
std::string			g_agent_context = "from-internal";
std::string			g_userData = "123456";

std::string			g_recording_filename ="ActivaTSPTest";
std::string			g_recording_format ="wav";
std::string			g_recording_mix ="true";

DWORD				g_timeout = 15;

DWORD				g_lastCallId = 0;

BasicLogger			g_logger("c:\\activa.log") ;

// Forward declarations of functions included in this code module:
bool				AST_Connect(std::string ip, std::string login, std::string pwd );
bool				AST_Disconnect();
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	InputDNDialog(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	InputIPDialog(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	MakeCallDialog(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	MakePredictiveCallDialog(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	QueueAddDialog(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	TransferCallDialog(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	StartRecordingDialog(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	StopRecordingDialog(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	PauseRecordingDialog(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	UnpauseRecordingDialog(HWND, UINT, WPARAM, LPARAM);

void				Trace(std::string msg);

bool				AST_OpenMonitor(std::string dn);
bool				AST_CloseMonitor(MonitorXref xref);
bool				AST_MakeCall(std::string from, std::string to, std::string context, std::string callerIdName);
bool				AST_MakePredictiveCall(std::string from, std::string contextFrom, std::string to, std::string contextTo, std::string userData, DWORD timeout);
bool				AST_ClearCall(DWORD callId, std::string address);
bool				AST_FastTransfer(DWORD callId, std::string from, std::string to, std::string context);
bool				AST_HoldCall(DWORD callId, std::string holdingDevice, std::string context);
bool				AST_retrieveCall(DWORD callId, std::string retrieveDevice, std::string context);	//returned the last call
bool				AST_QueueAdd(std::string extension, std::string queue, std::string context);
bool				AST_QueueRemove(std::string extension, std::string queue, std::string context);
bool				AST_QueuePause(std::string extension, std::string queue, bool pause, std::string context);
bool				AST_StartRecording(DWORD callid, std::string address, std::string RecordingFileName, std::string format, std::string mix);
bool				AST_StopRecording(DWORD callid, std::string address);
bool				AST_PauseRecording(DWORD callid, std::string address);
bool				AST_UnpauseRecording(DWORD callid, std::string address);

// listener
class myEventListener : public ASTCstaListener {
	void	OnOriginated ( MonitorXref xref, DWORD callId, std::string originatingDevice, std::string callerId, std::string callerIdName, std::string calledDevice, std::string userData, call_cause cause)
	{
		g_lastCallId = callId;

		Trace(GetTime()+" ORIGINATED:");
		Trace("  xref="+ToString(xref));
		Trace("  callid="+ToString(callId));
		Trace("  device="+originatingDevice);
	}

	void	OnAlerting ( MonitorXref xref, DWORD callId, std::string alertingDevice, std::string callerId, std::string callerIdName, std::string calledDevice, std::string lastRedirectingDevice, std::string userData, call_cause cause)
	{
		g_lastCallId = callId;

		Trace(GetTime()+" ALERTING:");
		Trace("  xref= "+ToString(xref));
		Trace("  callid= "+ToString(callId));
		Trace("  device= "+alertingDevice);
		Trace("  callerId= "+callerId);
		Trace("  callerIdName= "+callerIdName);
		Trace("  calledDevice= "+calledDevice);
		Trace("  lastRedirection= "+lastRedirectingDevice);
		Trace("  userData="+userData);
		Trace("  cause= "+ToString(cause));
	}

	void	OnFailed ( MonitorXref xref, DWORD callId, std::string failingDevice, std::string callerId, std::string callerIdName, std::string calledDevice, std::string lastRedirectionDevice, std::string userData, call_cause cause)
	{
		g_lastCallId = callId;

		Trace(GetTime()+" FAILED");
		Trace("  xref= "+ToString(xref));
		Trace("  callid= "+ToString(callId));
		Trace("  device= "+failingDevice);
		Trace("  callerId= "+callerId);
		Trace("  callerIdName= "+callerIdName);
		Trace("  calledDevice= "+calledDevice);
		Trace("  cause= "+ToString(cause));
	}

	void	OnEstablished ( MonitorXref xref, DWORD callId, std::string establishedDevice, std::string callerId, std::string callerIdName, std::string calledDevice, std::string lastRedirectingDevice, std::string userData, call_cause cause)
	{
		g_lastCallId = callId;

		Trace(GetTime()+" ESTABLISHED");
		Trace("  xref= "+ToString(xref));
		Trace("  callid= "+ToString(callId));
		Trace("  device= "+establishedDevice);
		Trace("  callerId= "+callerId);
		Trace("  callerIdName= "+callerIdName);
		Trace("  calledDevice= "+calledDevice);
		Trace("  lastRedirection= "+lastRedirectingDevice);
		Trace("  userData="+userData);
		Trace("  cause= "+ToString(cause));
	}

	void	OnConnectionCleared ( MonitorXref xref, DWORD callId, std::string releasingDevice, call_cause cause)
	{
		g_lastCallId = 0;

		Trace(GetTime()+" CONN_CLEARED");
		Trace("  xref= "+ToString(xref));
		Trace("  callid= "+ToString(callId));
		Trace("  device= "+releasingDevice);
		Trace("  cause= "+ToString(cause));
	}

	void	OnCallCleared ( MonitorXref xref, DWORD callId, std::string userData, call_cause cause)
	{
		g_lastCallId = 0;

		Trace(GetTime()+" CALL_CLEARED");
		Trace("  xref= "+ToString(xref));
		Trace("  callid= "+ToString(callId));
		Trace("  userData="+userData);
		Trace("  cause= "+ToString(cause));
	}

	void	OnQueued ( MonitorXref xref, DWORD callId, std::string queuedDevice, std::string queue, int position_in_queue, int queue_len, std::string callerId, std::string callerIdName, std::string calledDevice, std::string userData, call_cause cause) 
	{
		g_lastCallId = callId;

		Trace(GetTime()+" QUEUED");
		Trace("  xref= "+ToString(xref));
		Trace("  callid= "+ToString(callId));
		Trace("  device= "+queuedDevice);
		Trace("  queue= "+queue);
		Trace("    at= "+ToString(position_in_queue));
		Trace("      queue_len= "+ToString(queue_len));
		Trace("  callerId= "+callerId);
		Trace("  callerIdName= "+callerIdName);
		Trace("  calledDevice= "+calledDevice);
		Trace("  userData="+userData);
		Trace("  cause= "+ToString(cause));
	}

	void	OnDiverted ( MonitorXref xref, DWORD callId, std::string divertedDevice, std::string divertingDevice, std::string callerId, std::string callerIdName, std::string calledDevice, std::string userData, call_cause cause) 
	{
		g_lastCallId = callId;

		Trace(GetTime()+" DIVERTED");
		Trace("  xref= "+ToString(xref));
		Trace("  callid= "+ToString(callId));
		Trace("  device= "+divertedDevice);
		Trace("  diverting device= "+divertingDevice);
		Trace("  callerId= "+callerId);
		Trace("  callerIdName= "+callerIdName);
		Trace("  calledDevice= "+calledDevice);
		Trace("  cause= "+ToString(cause));
	}

	void	OnLinkDown()
	{
		Trace(GetTime()+" LINK DOWN!!!");
		Trace("============");
	}

	void	OnHold(MonitorXref xref, DWORD callId, std::string holdingDevice, call_cause cause)
	{
		g_lastCallId = callId;

		Trace(GetTime()+ " HELD");
		Trace("  xref= "+ToString(xref));
		Trace("  callid= "+ToString(callId));
		Trace("  device= "+holdingDevice);
		Trace("  cause= "+ToString(cause));
	}

	void	OnUnhold(MonitorXref xref, DWORD callId, std::string unholdingDevice, call_cause cause)
	{
		g_lastCallId = callId;

		Trace(GetTime()+" ESTABLISHED");
		Trace("  xref= "+ToString(xref));
		Trace("  callid= "+ToString(callId));
		Trace("  device= "+unholdingDevice);
		Trace("  cause= "+ToString(cause));
	}

	std::string	GetTime() {
		__time64_t ltime;

		_time64( &ltime );
		char buf[SIZE];
		errno_t err = _ctime64_s( buf,SIZE,&ltime );
		std::string now = ToString(buf);
		now = now.substr(0, now.size()-1);
		return now;
	};
};


//
int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_ASTPROVIDERTEST, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_ASTPROVIDERTEST);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	if (g_Provider) {
		//delete g_Provider;
		if(g_DeleteASTCSTAProvider!=NULL)
			(*g_DeleteASTCSTAProvider)(g_Provider);
		
	}

	FreeLibrary(g_hInstActivaAPI);

	if (g_Listener) {
		delete g_Listener;
	}

	return (int) msg.wParam;
}



/**
 * MyRegisterClass()
 *		Registers the window class.
 *		This function and its usage are only necessary if you want this code
 * to be compatible with Win32 systems prior to the 'RegisterClassEx'
 * function that was added to Windows 95. It is important to call this function
 * so that the application will get 'well formed' small icons associated
 * with it.
 */
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_ASTPROVIDERTEST);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= (LPCTSTR)IDC_ASTPROVIDERTEST;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

/**
 * InitInstance(HANDLE, int)
 */
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd;
    RECT rect;

	hInst = hInstance; // Store instance handle in our global variable

	hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
	      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

	GetClientRect(hWnd, &rect);

	g_hWndTraceWindow = CreateWindowEx(WS_EX_CLIENTEDGE,
							"EDIT",
							"",
							WS_BORDER|WS_VISIBLE|WS_CHILD|ES_MULTILINE|ES_AUTOVSCROLL|WS_VSCROLL|SS_LEFTNOWORDWRAP | ES_READONLY ,
							0,0,rect.right,rect.bottom,
							hWnd,
							NULL,
							hInst,
							NULL);

	if (!hWnd) {
		return FALSE;
	}

	g_Listener = new myEventListener();
	g_logger.EnableTraceToFile(true);
	//g_Provider = ASTCstaProvider::Create(g_Listener, &g_logger);
	

	g_hInstActivaAPI = LoadLibrary(ACTIVAAPI_LIBRARY);
	
	if (g_hInstActivaAPI > (HINSTANCE)HINSTANCE_ERROR)
	{
		g_CreateASTCSTAProvider = (LPFN_CreateASTCSTAProvider) GetProcAddress(g_hInstActivaAPI, "CreateASTCSTAProvider");

		LPVOID lpMsgBuf;
		DWORD dw = GetLastError(); 

		FormatMessage(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM |
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			dw,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			(LPTSTR) &lpMsgBuf,
			0, NULL );

		g_DeleteASTCSTAProvider = (LPFN_DeleteASTCSTAProvider) GetProcAddress(g_hInstActivaAPI, "DeleteASTCSTAProvider");
	
		if(g_CreateASTCSTAProvider!=NULL)
			g_Provider = ((*g_CreateASTCSTAProvider)(g_Listener, &g_logger));

	}
	if (!g_Provider) {
		return FALSE;
	}

	//g_Provider->SetLogListener(&g_logger);

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);
	g_hWnd = hWnd;

	return TRUE;
}

/**
 * WndProc(HWND, unsigned, WORD, LONG)
 *	Processes messages for the main window.
 */
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc;

    RECT rect;
	INT_PTR dw = 0;

	switch (message) 
	{
	case WM_COMMAND:
		wmId    = LOWORD(wParam); 
		wmEvent = HIWORD(wParam); 
		// Parse the menu selections:
		switch (wmId)
		{
		case IDM_ABOUT:
			DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
			break;

		case IDM_EXIT:
			DestroyWindow(hWnd);
			break;

		case ID_FILE_CONNECTTOASTERISK:
			dw = DialogBox(hInst, (LPCTSTR)IDD_IP_ADDRESS, hWnd, (DLGPROC)InputIPDialog);
			if (dw==IDOK) {
				if (!AST_Connect(g_ip, g_login, g_pwd)) {
					MessageBox(g_hWnd, "COULD NOT CONNECT", "connect to asterisk...", 0);
				}
			}
			break;

		case ID_FILE_DISCONNECT:
			if (AST_Disconnect()) {
				MessageBox(g_hWnd, "disconnected!!!", "connect to asterisk...", 0);
			}
			else {
				MessageBox(g_hWnd, "COULD NOT DISCONNECT", "connect to asterisk...", 0);
			}
			break;

		case ID_MONITORS_OPEN:
			dw = DialogBox(hInst, (LPCTSTR)IDD_DN, hWnd, (DLGPROC)InputDNDialog);
			if (dw==IDOK) {
				AST_OpenMonitor(g_dn);
			}
			break;

		case ID_MONITORS_CLOSE:
			AST_CloseMonitor(g_xref);
			break;

		case ID_ASTERISK_MAKECALL:
			dw = DialogBox(hInst, (LPCTSTR)IDD_MAKECALL, hWnd, (DLGPROC)MakeCallDialog);
			if (dw==IDOK) {
				AST_MakeCall(g_dn, g_to, g_outbound_context, g_calleridname);
			}
			break;

		case ID_ASTERISK_MAKEPREDICTIVECALL:
			dw = DialogBox(hInst, (LPCTSTR)IDD_MAKEPREDICTIVECALL, hWnd, (DLGPROC)MakePredictiveCallDialog);
			if (dw==IDOK){
				AST_MakePredictiveCall(g_dn, g_incoming_context, g_to, g_outbound_context, g_userData, g_timeout);
			}
			break;
		case ID_ASTERISK_DROPCALL:
			AST_ClearCall(g_lastCallId, g_dn);
			break;

		case ID_ASTERISK_HOLDCALL:
			dw = DialogBox(hInst, (LPCTSTR)IDD_DN, hWnd, (DLGPROC)InputDNDialog);
			if (dw == IDOK){
                AST_HoldCall(g_lastCallId, g_dn, g_outbound_context);
			}
			break;
		case ID_ASTERISK_TRANSFER:
			dw = DialogBox(hInst, (LPCTSTR)IDD_TRANSFER, hWnd, (DLGPROC) TransferCallDialog);
			if (dw==IDOK){
			}
			break;
		case ID_ASTERISK_RETRIEVECALL:
			dw = DialogBox(hInst, (LPCTSTR)IDD_DN, hWnd, (DLGPROC)InputDNDialog);
			if (dw == IDOK){
				AST_retrieveCall(g_lastCallId, g_dn, g_outbound_context);
			}
			break;
		case ID_ASTERISK_FASTTRANSFER:
			dw = DialogBox(hInst, (LPCTSTR)IDD_TRANSFER, hWnd, (DLGPROC)TransferCallDialog);
			if (dw==IDOK) {
				AST_FastTransfer(g_lastCallId, g_dn, g_to, g_outbound_context);
			}
			break;
		case ID_ASTERISK_QUEUEADD:
			dw = DialogBox(hInst, (LPCTSTR)IDD_QUEUEADD, hWnd, (DLGPROC)QueueAddDialog);
			if (dw==IDOK) {
				AST_QueueAdd(g_dn, g_queue, g_agent_context);
			}
			break;
		case ID_ASTERISK_QUEUEREMOVE:
			dw = DialogBox(hInst, (LPCTSTR)IDD_QUEUEADD, hWnd, (DLGPROC)QueueAddDialog);
			if (dw==IDOK) {
				AST_QueueRemove(g_dn, g_queue, g_agent_context);
			}
			break;
		case ID_ASTERISK_QUEUEPAUSE:
			dw = DialogBox(hInst, (LPCTSTR)IDD_DN, hWnd, (DLGPROC)InputDNDialog);
			if (dw==IDOK) {
				AST_QueuePause(g_dn, g_queue, true, g_agent_context);
			}
			break;
		case ID_ASTERISK_QUEUEPAUSE_NO:
			dw = DialogBox(hInst, (LPCTSTR)IDD_DN, hWnd, (DLGPROC)InputDNDialog);
			if (dw==IDOK) {
				AST_QueuePause(g_dn, g_queue, false, g_agent_context);
			}
			break;
		case ID_ASTERISK_STARTRECORDING:
			dw = DialogBox(hInst, (LPCTSTR)IDD_STARTRECORDING, hWnd, (DLGPROC)StartRecordingDialog);
			if (dw==IDOK) {
				AST_StartRecording(g_lastCallId, g_dn, g_recording_filename, g_recording_format, g_recording_mix);
			}
			break;
		case ID_ASTERISK_STOPRECORDING:
				AST_StopRecording(g_lastCallId, g_dn);
			break;
		case ID_ASTERISK_PAUSERECORDING:
				AST_PauseRecording(g_lastCallId, g_dn);
			break;
		case ID_ASTERISK_UNPAUSERECORDING:
				AST_UnpauseRecording(g_lastCallId, g_dn);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		EndPaint(hWnd, &ps);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	case WM_SIZE:
		if (wParam != SIZE_MINIMIZED) {
			GetClientRect(hWnd, &rect);
			//Resize 
			MoveWindow(g_hWndTraceWindow, 0, //x
						0, //y
						rect.right, // w
						rect.bottom, //h 
						TRUE);
			InvalidateRect(g_hWndTraceWindow, &rect, TRUE);
			UpdateWindow(g_hWndTraceWindow);
			UpdateWindow(g_hWnd);
		}
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// Message handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}

// Message handler for InputDNDialog box.
LRESULT CALLBACK InputDNDialog(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowText( GetDlgItem(hDlg,IDC_EDIT1), g_dn.c_str()); 
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK) 
		{

			HWND hEdit = ::GetDlgItem(hDlg,IDC_EDIT1); 
            if ( hEdit ) 
            { 
                // get the input text the user entered 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_dn = pText;
			}
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		else if (LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}


// Message handler for MakeCallDialog box.
LRESULT CALLBACK MakeCallDialog(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowText( GetDlgItem(hDlg,IDC_FROM), g_dn.c_str()); 
		SetWindowText( GetDlgItem(hDlg,IDC_TO), g_to.c_str()); 
		SetWindowText( GetDlgItem(hDlg,IDC_CALLERIDNAME), g_calleridname.c_str()); 
		SetWindowText( GetDlgItem(hDlg,IDC_OUTBOUNDCONTEXT), g_outbound_context.c_str()); 
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK) {	// get the input text the user entered 
			HWND hEdit;
			hEdit = ::GetDlgItem(hDlg,IDC_FROM); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_dn = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_TO); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_to = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_CALLERIDNAME); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_calleridname = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_OUTBOUNDCONTEXT); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_outbound_context = pText;
			}
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		else if (LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}

// Message handler for MakePredictiveCall Dialog
LRESULT CALLBACK MakePredictiveCallDialog(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowText(GetDlgItem(hDlg, IDC_FROM), g_dn.c_str());
		SetWindowText(GetDlgItem(hDlg, IDC_FROMCONTEXT), g_incoming_context.c_str());
		SetWindowText(GetDlgItem(hDlg, IDC_TO), g_to.c_str());
		SetWindowText(GetDlgItem(hDlg, IDC_TIMEOUT), ToString(g_timeout).c_str());
		SetWindowText(GetDlgItem(hDlg, IDC_EXTENCONTEXT), g_outbound_context.c_str());
		SetWindowText(GetDlgItem(hDlg, IDC_USERDATA), g_userData.c_str());
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK){
			HWND hEdit = ::GetDlgItem(hDlg,IDC_FROM); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_dn = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_FROMCONTEXT); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_incoming_context = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_TO); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_to = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_TIMEOUT); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_timeout = atoi(pText);
			}
			hEdit = ::GetDlgItem(hDlg,IDC_EXTENCONTEXT); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_outbound_context = pText;
			}
			hEdit = ::GetDlgItem(hDlg, IDC_USERDATA);
			if (hEdit){
				char pText[1024];
				int nSize = ::GetWindowText(hEdit,pText,1023);
				pText[nSize] = 0;
				g_userData = pText;
			}
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		else if (LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}

// Message handler for InputIPDialog box.
LRESULT CALLBACK InputIPDialog(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowText( GetDlgItem(hDlg,IDC_EDIT1), g_ip.c_str()); 
        SetWindowText( GetDlgItem(hDlg,IDC_LOGIN), g_login.c_str()); 
        SetWindowText( GetDlgItem(hDlg,IDC_PWD), g_pwd.c_str()); 
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK) {
            // get the input text the user entered 
			HWND hEdit = ::GetDlgItem(hDlg,IDC_EDIT1); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_ip = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_LOGIN); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_login = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_PWD); 
            if ( hEdit ) { 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_pwd = pText;
			}
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		else if (LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}

// Message handler for QueueAddDialog box.
LRESULT CALLBACK QueueAddDialog(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowText( GetDlgItem(hDlg,IDC_EXTENSION), g_dn.c_str()); 
		SetWindowText( GetDlgItem(hDlg,IDC_QUEUE), g_queue.c_str()); 
		SetWindowText( GetDlgItem(hDlg,IDC_CONTEXT), g_agent_context.c_str());
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK) {
			HWND hEdit ;
			hEdit = ::GetDlgItem(hDlg,IDC_EXTENSION); 
            if ( hEdit ) { 
                // get the input text the user entered 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_dn = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_QUEUE); 
            if ( hEdit ) { 
                // get the input text the user entered 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_queue = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_CONTEXT); 
            if ( hEdit ) { 
                // get the input text the user entered 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_agent_context = pText;
			}
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		else if (LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}

LRESULT CALLBACK	TransferCallDialog(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowText( GetDlgItem(hDlg,IDC_FROM), g_dn.c_str()); 
		SetWindowText( GetDlgItem(hDlg,IDC_TO), g_to.c_str()); 
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK) {
			HWND hEdit ;
			hEdit = ::GetDlgItem(hDlg,IDC_FROM); 
            if ( hEdit ) { 
                // get the input text the user entered 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_dn = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_TO); 
            if ( hEdit ) { 
                // get the input text the user entered 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_to = pText;
			}
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		else if (LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}

LRESULT CALLBACK	StartRecordingDialog(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowText( GetDlgItem(hDlg,IDC_FILENAME), g_recording_filename.c_str()); 
		SetWindowText( GetDlgItem(hDlg,IDC_FORMAT), g_recording_format.c_str()); 
		SetWindowText( GetDlgItem(hDlg,IDC_MIX), g_recording_format.c_str());
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK) {
			HWND hEdit ;
			hEdit = ::GetDlgItem(hDlg,IDC_FILENAME); 
            if ( hEdit ) { 
                // get the input text the user entered 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_recording_filename = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_FORMAT); 
            if ( hEdit ) { 
                // get the input text the user entered 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_recording_format = pText;
			}
			hEdit = ::GetDlgItem(hDlg,IDC_MIX); 
            if ( hEdit ) { 
                // get the input text the user entered 
                char pText[1024]; 
                int nSize = ::GetWindowText( hEdit, pText, 1023 ); 
                pText[nSize] = 0; 
				g_recording_format = pText;
			}
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		else if (LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}


std::string  buffer;

void Trace(std::string msg)
{
	buffer += msg;
	buffer += "\r\n";
	SetWindowText(g_hWndTraceWindow, (LPCSTR)buffer.c_str());
	SendMessage(g_hWndTraceWindow, EM_LINESCROLL, 0, 1);
	int	minpos, maxpos;
	GetScrollRange(g_hWndTraceWindow, SB_VERT, &minpos, &maxpos);
	SetScrollPos(g_hWndTraceWindow,	SB_VERT, maxpos, TRUE);

	RECT	rect;
	GetClientRect(g_hWndTraceWindow, &rect);
	InvalidateRect(g_hWndTraceWindow, &rect, TRUE);
	UpdateWindow(g_hWndTraceWindow);
}


/******************************************************************
** testing functions
*/
bool	AST_Connect(std::string ip, std::string login, std::string pwd)
{
	if (!g_Provider) {
		return false;
	}

	if ( ok == g_Provider->Connect(ip, 5038, login, pwd, at_md5)) {
		Trace("connected to asterisk... ip="+ip+" user/pwd="+login+"/"+pwd);
		return true;
	}
	return false;
}

bool	AST_Disconnect()
{
	if (!g_Provider) {
		return false;
	}

	if(g_Provider->Disconnect()) {
		Trace("disconnected from asterisk...");
		return true;
	}
	return false;

}

bool	AST_OpenMonitor(std::string dn)
{
	if (!g_Provider) {
		return false;
	}
	//if (g_xref>0)
	//	g_Provider->CloseMonitor(g_xref);

	g_xref = g_Provider->OpenMonitor(dn);
	Trace("Monitor opened -> "+dn+" xref = "+ToString(g_xref));
	
	return g_xref>0?true:false;
}

bool	AST_CloseMonitor(MonitorXref xref)
{
	if (!g_Provider) {
		return false;
	}
	if (g_Provider->CloseMonitor(xref)) {
		Trace("Monitor closed -> xref = "+ToString(xref));
		return true;
	}
	return false;
}


bool	AST_MakeCall(std::string from, std::string to, std::string context ,std::string calledIdName)
{
	if (!g_Provider) {
		return false;
	}

	DWORD callId;
	AC_RESULT result = g_Provider->MakeCall(from, context, to, callId, context, calledIdName,"","");
	Trace("MakeCall returned rc= "+ToString(result));
	
	return true;
}

bool	AST_MakePredictiveCall(std::string from, std::string contextFrom, std::string to, std::string contextTo, std::string userData, DWORD timeout)
{
	if (!g_Provider){
		return false;
	}
	DWORD callId;
	std::string callerid = to;
	AC_RESULT result = g_Provider->MakePredictiveCall(from, contextFrom, to, contextTo, callId, callerid, userData, timeout,"");
	//callId = g_Provider->MakePredictiveCall(from, to, context, callerIdName);
	Trace("MakePredictiveCall returned rc="+ToString(result));

	return true;
}

bool	AST_ClearCall(DWORD callId, std::string address)
{
	if (!g_Provider) {
		return false;
	}

	AC_RESULT result = g_Provider->ClearCall(callId);
	Trace("ClearCall returned rc= "+ToString(result));
	
	return true;
}

bool	AST_FastTransfer(DWORD callId, std::string from, std::string to, std::string context)
{
	if (!g_Provider) {
		return false;
	}

	AC_RESULT result = g_Provider->FastTransfer(callId, from, to, context);
	Trace("FastTransfer returned rc= "+ToString(result));
	
	return true;
}
bool	AST_HoldCall(DWORD callId, std::string holdingDevice, std::string holdContext)
{
	if (!g_Provider){
		return false;
	}
	AC_RESULT result = g_Provider->HoldCall(callId, holdingDevice, holdContext);
	Trace("HoldCall returned rc="+ToString(result));
	return true;
}
bool	AST_retrieveCall(DWORD callId, std::string retrieveDevice, std::string retrieveContext)
{
	if (!g_Provider){
		return false;
	}
	AC_RESULT result = g_Provider->RetrieveCall(callId, retrieveDevice, retrieveContext);
	Trace("retrieveCall returned rc="+ToString(result));
	return true;
}

bool	AST_QueueAdd(std::string extension, std::string queue, std::string context)
{
	if (!g_Provider) {
		return false;
	}

	AC_RESULT result = g_Provider->SetAgentStatus(extension, queue, LOGIN, context);
	Trace("SetAgentStatus(LOGIN) returned rc= "+ToString(result));
	
	return true;
}

bool	AST_QueueRemove(std::string extension, std::string queue, std::string context)
{
	if (!g_Provider) {
		return false;
	}

	AC_RESULT result = g_Provider->SetAgentStatus(extension, queue, LOGOUT, context);
	Trace("SetAgentStatus(LOGOUT) returned rc= "+ToString(result));
	
	return true;
}

bool	AST_QueuePause(std::string extension, std::string queue, bool pause, std::string context)
{
	if (!g_Provider) {
		return false;
	}

	if (pause) {
		AC_RESULT result = g_Provider->SetAgentStatus(extension, queue, NOTREADY, context);
		Trace("SetAgentStatus(NOTREADY) returned rc= "+ToString(result));
	}
	else {
		AC_RESULT result = g_Provider->SetAgentStatus(extension, queue, READY, context);
		Trace("SetAgentStatus(READY) returned rc= "+ToString(result));
	}
	
	return true;
}

bool	AST_StartRecording(DWORD callid, std::string address, std::string RecordingFileName, std::string format, std::string mix)
{
	if (!g_Provider) {
		return false;
	}
	
	AC_RESULT result = g_Provider->StartRecording(callid,address,RecordingFileName,format,mix);
	Trace("StartRecording returned rc= "+ToString(result));
	
	return true;
}

bool	AST_StopRecording(DWORD callid, std::string address)
{
	if (!g_Provider) {
		return false;
	}

	AC_RESULT result = g_Provider->StopRecording(callid,address);
	Trace("StopRecording returned rc= "+ToString(result));
	
	return true;
}

bool	AST_PauseRecording(DWORD callid, std::string address)
{
	if (!g_Provider) {
		return false;
	}

	AC_RESULT result = g_Provider->PauseRecording(callid,address);
	Trace("PauseRecording returned rc= "+ToString(result));
	
	return true;
}

bool	AST_UnpauseRecording(DWORD callid, std::string address)
{
	if (!g_Provider) {
		return false;
	}

	AC_RESULT result = g_Provider->UnpauseRecording(callid,address);
	Trace("UnpauseRecording returned rc= "+ToString(result));
	
	return true;
}