/*
	Name: installActivaTSP.cpp
	Copyright: Under the Mozilla Public License Version 1.1 or later
	Author: ACTIVA
	Date: MAY/06
*/

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is astloader.
 *
 * The Initial Developer of the Original Code is Nick Knight.
 *
 * Contributor(s): Activa
 *
 * ***** END LICENSE BLOCK ***** */

#include "stdafx.h"
#include "installActivaTSP.h"
#include <tapi.h>
#include <string>

#define MAX_LOADSTRING 100


//Thanks to 
// "Windows Telephony Programming - A developers guide to TAPI"
// by Chris Sells for functions.
bool GetPermanentProviderID(LPCSTR pszProvider, DWORD* pdwID);
LONG GetProviderList(LINEPROVIDERLIST**  ppd);
LONG ShowConfig();


int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	LPCSTR pszProvider = "ActivaTSP.tsp";
	LPCSTR pszTitle = "CallBox TSP Installation";
	LPCSTR pszMessage = 0;
	int ret=0;

	std::string	cmdLine = lpCmdLine;
	const std::string installCmd = "/install:";
	std::string::size_type i = cmdLine.find(installCmd, 0);
	std::string path;
	if(i!=-1) {
		path = cmdLine.substr(i+installCmd.size());
		pszProvider = path.c_str();
	}

	//mark to config dialog
	BOOL bConf=false;
	if(cmdLine.find("/config", 0)!=-1){
		if(cmdLine.find("/install", 0)!=-1) { //mark to config dialog after install
			bConf=true;
		}else{ //config dialog and exit
			return ShowConfig();
		}
	}
	
    DWORD dwID;
	LONG retVal ;
	//first remove
    if( GetPermanentProviderID(pszProvider, &dwID) ) {
		retVal = lineRemoveProvider(dwID, 0);
        if( retVal == 0 ) {
            pszMessage = "CallBox TSP removed.";
        }
        else {
            pszMessage = "CallBox TSP cannot be removed.";
			ret=-1;
        }
    }
    else  {
        pszMessage = "CallBox TSP not installed.";
    }

	if (_strcmpi(lpCmdLine, "/removesilently") == 0) {
        pszMessage = NULL;
	}

    // Add the TSP if !remove
	if( !((_strcmpi(lpCmdLine, "/remove") == 0 ) || (_strcmpi(lpCmdLine, "/removesilently") == 0 ))) {
	    DWORD   dwID;
		LONG retVal = lineAddProvider(pszProvider, 0, &dwID);
        switch( retVal ) {
        case 0:
			pszMessage = "CallBox TSP successfully installed. Windows must be restarted.";
        break;

        case LINEERR_NOMULTIPLEINSTANCE:
            pszMessage = "CallBox TSP already installed.";
        break;
		
		case LINEERR_INIFILECORRUPT:
			pszMessage = "Unable to install CallBox TSP: ini file corrupt";
			ret=-1;
			break;
		case LINEERR_NOMEM:
			pszMessage = "Unable to install CallBox TSP: no mem";
			ret=-1;
			break;
		case LINEERR_INVALPARAM:
			pszMessage = "Unable to install CallBox TSP: invalid param";
			ret=-1;
			break;
		case LINEERR_INVALPOINTER:
			pszMessage = "Unable to install CallBox TSP: invalid pointer";
			ret=-1;
			break;
		case LINEERR_OPERATIONFAILED:
			pszMessage = "Unable to install CallBox TSP: operation failed";
			ret=-1;
			break;
		case LINEERR_OPERATIONUNAVAIL:
			pszMessage = "Unable to install CallBox TSP: operation unavailable";
			ret=-1;
			break;

        default:
			pszMessage = "Unable to install CallBox TSP";
			ret=-1;
        break;
        }
		
		if (_strcmpi(lpCmdLine, "/installsilently") == 0) {
            pszMessage = NULL;
		}

	}

    if( pszMessage ) {
        MessageBox(0, pszMessage, pszTitle, MB_SETFOREGROUND);
    }

	if (bConf) ShowConfig();

	return (int)ret;
}


// Caller responsible for freeing returned memory
LONG GetProviderList(LINEPROVIDERLIST**  ppd)
{
    // Load tapi32.dll explictly. This won't matter
    // for clients that already have tapi32.dll loaded,
    // but makes all the difference if they don't.
    HINSTANCE   hinstDll = 0;
    LONG        (WINAPI* pfnGetProviderList)(DWORD, LINEPROVIDERLIST*) = 0;

    if( hinstDll = LoadLibrary("tapi32.dll") ) {
        *(FARPROC*)&pfnGetProviderList = GetProcAddress(hinstDll, "lineGetProviderList");

        if( !pfnGetProviderList ) {
            FreeLibrary(hinstDll);
            return LINEERR_OPERATIONFAILED;
        }
    }
    else {
        return LINEERR_OPERATIONFAILED;
    }

    // Fill LINEPROVIDERLIST
    DWORD dwNeededSize = sizeof(LINEPROVIDERLIST);
    LONG  tr = 0;

    do {
        // Get some more memory if we don't have enough
        if( !*ppd || (*ppd)->dwTotalSize < dwNeededSize ) {
            *ppd = (LINEPROVIDERLIST*)::realloc(*ppd, dwNeededSize);
            if( *ppd ) {
                (*ppd)->dwTotalSize = dwNeededSize;
            }
            else {
                return LINEERR_NOMEM;
            }
        }

        // Fill in the buffer
        tr = pfnGetProviderList(TAPI_CURRENT_VERSION, *ppd);

        // Check how much memory we need
        // (some TSPs succeed even if the
        //  data size is too small)
        if( tr == LINEERR_STRUCTURETOOSMALL ||
            (tr == 0 &&
             (*ppd)->dwTotalSize < (*ppd)->dwNeededSize) ) {
            dwNeededSize = (*ppd)->dwNeededSize;
            tr = LINEERR_STRUCTURETOOSMALL;
        }
    } while( tr == LINEERR_STRUCTURETOOSMALL );

    // Unload tapi32.dll (or just release our reference to it)
    FreeLibrary(hinstDll);

    return tr;
}

bool GetPermanentProviderID(LPCSTR pszProvider, DWORD* pdwID)
{
    bool    bSucceeded = false;

    LINEPROVIDERLIST*   ppl = 0;
	if( GetProviderList(&ppl) == 0 ) {
        LPLINEPROVIDERENTRY rglpe = (LPLINEPROVIDERENTRY)((BYTE*)ppl + ppl->dwProviderListOffset);
        for( DWORD i = 0; i < ppl->dwNumProviders; i++ ) {
            LPCSTR pszProviderFilename = (LPCSTR)((BYTE*)ppl + rglpe[i].dwProviderFilenameOffset);

            if( _strcmpi(pszProviderFilename, pszProvider) == 0 ) {
                *pdwID = rglpe[i].dwPermanentProviderID;
                bSucceeded = true;
                break;
            }
        }

        free(ppl);
    }

    return bSucceeded;
}


LONG ShowConfig(){

	HLINEAPP		hLineApp=0;
	DWORD			version=TAPI_CURRENT_VERSION;
	DWORD			dwNumDevs;
	LONG			A;
	LINEINITIALIZEEXPARAMS	stExtParams;

	memset(&stExtParams, 0, sizeof(LINEINITIALIZEEXPARAMS));
	stExtParams.dwOptions = LINEINITIALIZEEXOPTION_USEEVENT;//LINEINITIALIZEEXOPTION_USEHIDDENWINDOW;
	stExtParams.dwTotalSize = sizeof(LINEINITIALIZEEXPARAMS);
	
	A = lineInitializeEx(&hLineApp,NULL,NULL,"registerActivaTSP",&dwNumDevs,&version,&stExtParams);
	if (A!=0) return A;
	//A short alternative to show dialog after install without any check
	//A=lineConfigDialog(dwNumDevs-1 ,0,0);
	
	LINEDEVCAPS   *lpLineDevCaps;
	char buffer [1024];
	lpLineDevCaps = (LINEDEVCAPS*)buffer;
	std::string providerInfo = "CallBox TSP";	
	lpLineDevCaps->dwTotalSize=sizeof(buffer);
	char* TSP_INFO;
	DWORD dwDev=dwNumDevs;
	BOOL bShowed = false;
	while (dwDev-- > 0 && !bShowed ){
		A = lineGetDevCaps(hLineApp, dwDev, version, 0, lpLineDevCaps);	
		if(A==0 && lpLineDevCaps->dwTotalSize > lpLineDevCaps->dwNeededSize && lpLineDevCaps->dwProviderInfoSize >= providerInfo.size()){
			TSP_INFO = (char*)(lpLineDevCaps);
			TSP_INFO=TSP_INFO+lpLineDevCaps->dwProviderInfoOffset;
			if (providerInfo.compare(TSP_INFO)==0){
				A=lineConfigDialog(dwDev ,0,0);
				bShowed=true;
			}
		}
	}
	//if config is not showed activaTSP not installed
	if (!bShowed) MessageBox(0, "CallBox TSP not installed.", "CallBox TSP Configuration", MB_SETFOREGROUND);

	A=lineShutdown(hLineApp);
	return A;
}